@@ -5,17 +5,20 @@ import numpy as np
 
 from .. import Variable
 from ..core import indexing
-from ..core.utils import FrozenOrderedDict, close_on_error
+from ..core.utils import FrozenOrderedDict
 from .common import WritableCFDataStore
 from .file_manager import CachingFileManager
 from .locks import HDF5_LOCK, combine_locks, ensure_lock, get_write_lock
 from .netCDF4_ import (
-    BaseNetCDF4Array, _encode_nc4_variable, _extract_nc4_variable_encoding,
-    _get_datatype, _nc4_require_group)
+    BaseNetCDF4Array,
+    _encode_nc4_variable,
+    _extract_nc4_variable_encoding,
+    _get_datatype,
+    _nc4_require_group,
+)
 
 
 class H5NetCDFArrayWrapper(BaseNetCDF4Array):
-
     def get_array(self, needs_lock=True):
         ds = self.datastore._acquire(needs_lock)
         variable = ds.variables[self.variable_name]
@@ -23,8 +26,8 @@ class H5NetCDFArrayWrapper(BaseNetCDF4Array):
 
     def __getitem__(self, key):
         return indexing.explicit_indexing_adapter(
-            key, self.shape, indexing.IndexingSupport.OUTER_1VECTOR,
-            self._getitem)
+            key, self.shape, indexing.IndexingSupport.OUTER_1VECTOR, self._getitem
+        )
 
     def _getitem(self, key):
         # h5py requires using lists for fancy indexing:
@@ -37,7 +40,7 @@ class H5NetCDFArrayWrapper(BaseNetCDF4Array):
 
 def maybe_decode_bytes(txt):
     if isinstance(txt, bytes):
-        return txt.decode('utf-8')
+        return txt.decode("utf-8")
     else:
         return txt
 
@@ -48,15 +51,15 @@ def _read_attributes(h5netcdf_var):
     # bytes attributes to strings
     attrs = OrderedDict()
     for k, v in h5netcdf_var.attrs.items():
-        if k not in ['_FillValue', 'missing_value']:
+        if k not in ["_FillValue", "missing_value"]:
             v = maybe_decode_bytes(v)
         attrs[k] = v
     return attrs
 
 
 _extract_h5nc_encoding = functools.partial(
-    _extract_nc4_variable_encoding,
-    lsd_okay=False, h5py_okay=True, backend='h5netcdf')
+    _extract_nc4_variable_encoding, lsd_okay=False, h5py_okay=True, backend="h5netcdf"
+)
 
 
 def _h5netcdf_create_group(dataset, name):
@@ -67,18 +70,29 @@ class H5NetCDFStore(WritableCFDataStore):
     """Store for reading and writing data via h5netcdf
     """
 
-    def __init__(self, filename, mode='r', format=None, group=None,
-                 lock=None, autoclose=False):
+    def __init__(
+        self,
+        filename,
+        mode="r",
+        format=None,
+        group=None,
+        lock=None,
+        autoclose=False,
+        invalid_netcdf=None,
+    ):
         import h5netcdf
 
-        if format not in [None, 'NETCDF4']:
-            raise ValueError('invalid format for h5netcdf backend')
+        if format not in [None, "NETCDF4"]:
+            raise ValueError("invalid format for h5netcdf backend")
+
+        kwargs = {"invalid_netcdf": invalid_netcdf}
 
         self._manager = CachingFileManager(
-            h5netcdf.File, filename, mode=mode)
+            h5netcdf.File, filename, mode=mode, kwargs=kwargs
+        )
 
         if lock is None:
-            if mode == 'r':
+            if mode == "r":
                 lock = HDF5_LOCK
             else:
                 lock = combine_locks([HDF5_LOCK, get_write_lock(filename)])
@@ -92,8 +106,9 @@ class H5NetCDFStore(WritableCFDataStore):
 
     def _acquire(self, needs_lock=True):
         with self._manager.acquire_context(needs_lock) as root:
-            ds = _nc4_require_group(root, self._group, self._mode,
-                                    create_group=_h5netcdf_create_group)
+            ds = _nc4_require_group(
+                root, self._group, self._mode, create_group=_h5netcdf_create_group
+            )
         return ds
 
     @property
@@ -104,43 +119,43 @@ class H5NetCDFStore(WritableCFDataStore):
         import h5py
 
         dimensions = var.dimensions
-        data = indexing.LazilyOuterIndexedArray(
-            H5NetCDFArrayWrapper(name, self))
+        data = indexing.LazilyOuterIndexedArray(H5NetCDFArrayWrapper(name, self))
         attrs = _read_attributes(var)
 
         # netCDF4 specific encoding
         encoding = {
-            'chunksizes': var.chunks,
-            'fletcher32': var.fletcher32,
-            'shuffle': var.shuffle,
+            "chunksizes": var.chunks,
+            "fletcher32": var.fletcher32,
+            "shuffle": var.shuffle,
         }
         # Convert h5py-style compression options to NetCDF4-Python
         # style, if possible
-        if var.compression == 'gzip':
-            encoding['zlib'] = True
-            encoding['complevel'] = var.compression_opts
+        if var.compression == "gzip":
+            encoding["zlib"] = True
+            encoding["complevel"] = var.compression_opts
         elif var.compression is not None:
-            encoding['compression'] = var.compression
-            encoding['compression_opts'] = var.compression_opts
+            encoding["compression"] = var.compression
+            encoding["compression_opts"] = var.compression_opts
 
         # save source so __repr__ can detect if it's local or not
-        encoding['source'] = self._filename
-        encoding['original_shape'] = var.shape
+        encoding["source"] = self._filename
+        encoding["original_shape"] = var.shape
 
         vlen_dtype = h5py.check_dtype(vlen=var.dtype)
         if vlen_dtype is str:
-            encoding['dtype'] = str
+            encoding["dtype"] = str
         elif vlen_dtype is not None:  # pragma: no cover
             # xarray doesn't support writing arbitrary vlen dtypes yet.
             pass
         else:
-            encoding['dtype'] = var.dtype
+            encoding["dtype"] = var.dtype
 
         return Variable(dimensions, data, attrs, encoding)
 
     def get_variables(self):
-        return FrozenOrderedDict((k, self.open_store_variable(k, v))
-                                 for k, v in self.ds.variables.items())
+        return FrozenOrderedDict(
+            (k, self.open_store_variable(k, v)) for k, v in self.ds.variables.items()
+        )
 
     def get_attrs(self):
         return FrozenOrderedDict(_read_attributes(self.ds))
@@ -150,8 +165,9 @@ class H5NetCDFStore(WritableCFDataStore):
 
     def get_encoding(self):
         encoding = {}
-        encoding['unlimited_dims'] = {
-            k for k, v in self.ds.dimensions.items() if v is None}
+        encoding["unlimited_dims"] = {
+            k for k, v in self.ds.dimensions.items() if v is None
+        }
         return encoding
 
     def set_dimension(self, name, length, is_unlimited=False):
@@ -167,61 +183,71 @@ class H5NetCDFStore(WritableCFDataStore):
     def encode_variable(self, variable):
         return _encode_nc4_variable(variable)
 
-    def prepare_variable(self, name, variable, check_encoding=False,
-                         unlimited_dims=None):
+    def prepare_variable(
+        self, name, variable, check_encoding=False, unlimited_dims=None
+    ):
         import h5py
 
         attrs = variable.attrs.copy()
-        dtype = _get_datatype(
-            variable, raise_on_invalid_encoding=check_encoding)
+        dtype = _get_datatype(variable, raise_on_invalid_encoding=check_encoding)
 
-        fillvalue = attrs.pop('_FillValue', None)
+        fillvalue = attrs.pop("_FillValue", None)
         if dtype is str and fillvalue is not None:
             raise NotImplementedError(
-                'h5netcdf does not yet support setting a fill value for '
-                'variable-length strings '
-                '(https://github.com/shoyer/h5netcdf/issues/37). '
+                "h5netcdf does not yet support setting a fill value for "
+                "variable-length strings "
+                "(https://github.com/shoyer/h5netcdf/issues/37). "
                 "Either remove '_FillValue' from encoding on variable %r "
                 "or set {'dtype': 'S1'} in encoding to use the fixed width "
-                'NC_CHAR type.' % name)
+                "NC_CHAR type." % name
+            )
 
         if dtype is str:
             dtype = h5py.special_dtype(vlen=str)
 
-        encoding = _extract_h5nc_encoding(variable,
-                                          raise_on_invalid=check_encoding)
+        encoding = _extract_h5nc_encoding(variable, raise_on_invalid=check_encoding)
         kwargs = {}
 
         # Convert from NetCDF4-Python style compression settings to h5py style
         # If both styles are used together, h5py takes precedence
         # If set_encoding=True, raise ValueError in case of mismatch
-        if encoding.pop('zlib', False):
-            if (check_encoding and encoding.get('compression')
-                    not in (None, 'gzip')):
+        if encoding.pop("zlib", False):
+            if check_encoding and encoding.get("compression") not in (None, "gzip"):
                 raise ValueError("'zlib' and 'compression' encodings mismatch")
-            encoding.setdefault('compression', 'gzip')
+            encoding.setdefault("compression", "gzip")
 
-        if (check_encoding and
-                'complevel' in encoding and 'compression_opts' in encoding and
-                encoding['complevel'] != encoding['compression_opts']):
-            raise ValueError("'complevel' and 'compression_opts' encodings "
-                             "mismatch")
-        complevel = encoding.pop('complevel', 0)
+        if (
+            check_encoding
+            and "complevel" in encoding
+            and "compression_opts" in encoding
+            and encoding["complevel"] != encoding["compression_opts"]
+        ):
+            raise ValueError("'complevel' and 'compression_opts' encodings " "mismatch")
+        complevel = encoding.pop("complevel", 0)
         if complevel != 0:
-            encoding.setdefault('compression_opts', complevel)
+            encoding.setdefault("compression_opts", complevel)
 
-        encoding['chunks'] = encoding.pop('chunksizes', None)
+        encoding["chunks"] = encoding.pop("chunksizes", None)
 
         # Do not apply compression, filters or chunking to scalars.
         if variable.shape:
-            for key in ['compression', 'compression_opts', 'shuffle',
-                        'chunks', 'fletcher32']:
+            for key in [
+                "compression",
+                "compression_opts",
+                "shuffle",
+                "chunks",
+                "fletcher32",
+            ]:
                 if key in encoding:
                     kwargs[key] = encoding[key]
         if name not in self.ds:
             nc4_var = self.ds.create_variable(
-                name, dtype=dtype, dimensions=variable.dims,
-                fillvalue=fillvalue, **kwargs)
+                name,
+                dtype=dtype,
+                dimensions=variable.dims,
+                fillvalue=fillvalue,
+                **kwargs
+            )
         else:
             nc4_var = self.ds[name]
 
