@@ -1,28 +1,37 @@
-from __future__ import absolute_import, division, print_function
-
 import unicodedata
+from collections import OrderedDict
 
 import numpy as np
 
 from .. import Variable, coding
-from ..core.pycompat import OrderedDict, basestring, unicode_type
 
 # Special characters that are permitted in netCDF names except in the
 # 0th position of the string
-_specialchars = '_.@+- !"#$%&\()*,:;<=>?[]^`{|}~'
+_specialchars = '_.@+- !"#$%&\\()*,:;<=>?[]^`{|}~'
 
 # The following are reserved names in CDL and may not be used as names of
 # variables, dimension, attributes
-_reserved_names = set(['byte', 'char', 'short', 'ushort', 'int', 'uint',
-                       'int64', 'uint64', 'float' 'real', 'double', 'bool',
-                       'string'])
+_reserved_names = {
+    "byte",
+    "char",
+    "short",
+    "ushort",
+    "int",
+    "uint",
+    "int64",
+    "uint64",
+    "float" "real",
+    "double",
+    "bool",
+    "string",
+}
 
 # These data-types aren't supported by netCDF3, so they are automatically
 # coerced instead as indicated by the "coerce_nc3_dtype" function
-_nc3_dtype_coercions = {'int64': 'int32', 'bool': 'int8'}
+_nc3_dtype_coercions = {"int64": "int32", "bool": "int8"}
 
 # encode all strings as UTF-8
-STRING_ENCODING = 'utf-8'
+STRING_ENCODING = "utf-8"
 
 
 def coerce_nc3_dtype(arr):
@@ -41,8 +50,9 @@ def coerce_nc3_dtype(arr):
         # TODO: raise a warning whenever casting the data-type instead?
         cast_arr = arr.astype(new_dtype)
         if not (cast_arr == arr).all():
-            raise ValueError('could not safely cast array from dtype %s to %s'
-                             % (dtype, new_dtype))
+            raise ValueError(
+                "could not safely cast array from dtype %s to %s" % (dtype, new_dtype)
+            )
         arr = cast_arr
     return arr
 
@@ -50,7 +60,7 @@ def coerce_nc3_dtype(arr):
 def encode_nc3_attr_value(value):
     if isinstance(value, bytes):
         pass
-    elif isinstance(value, unicode_type):
+    elif isinstance(value, str):
         value = value.encode(STRING_ENCODING)
     else:
         value = coerce_nc3_dtype(np.atleast_1d(value))
@@ -60,13 +70,14 @@ def encode_nc3_attr_value(value):
 
 
 def encode_nc3_attrs(attrs):
-    return OrderedDict([(k, encode_nc3_attr_value(v))
-                        for k, v in attrs.items()])
+    return OrderedDict([(k, encode_nc3_attr_value(v)) for k, v in attrs.items()])
 
 
 def encode_nc3_variable(var):
-    for coder in [coding.strings.EncodedStringCoder(allows_unicode=False),
-                  coding.strings.CharacterArrayCoder()]:
+    for coder in [
+        coding.strings.EncodedStringCoder(allows_unicode=False),
+        coding.strings.CharacterArrayCoder(),
+    ]:
         var = coder.encode(var)
     data = coerce_nc3_dtype(var.data)
     attrs = encode_nc3_attrs(var.attrs)
@@ -79,7 +90,7 @@ def _isalnumMUTF8(c):
 
     Input is not checked!
     """
-    return c.isalnum() or (len(c.encode('utf-8')) > 1)
+    return c.isalnum() or (len(c.encode("utf-8")) > 1)
 
 
 def is_valid_nc3_name(s):
@@ -99,15 +110,17 @@ def is_valid_nc3_name(s):
     names. Names that have trailing space characters are also not
     permitted.
     """
-    if not isinstance(s, basestring):
+    if not isinstance(s, str):
         return False
-    if not isinstance(s, unicode_type):
-        s = s.decode('utf-8')
-    num_bytes = len(s.encode('utf-8'))
-    return ((unicodedata.normalize('NFC', s) == s) and
-            (s not in _reserved_names) and
-            (num_bytes >= 0) and
-            ('/' not in s) and
-            (s[-1] != ' ') and
-            (_isalnumMUTF8(s[0]) or (s[0] == '_')) and
-            all((_isalnumMUTF8(c) or c in _specialchars for c in s)))
+    if not isinstance(s, str):
+        s = s.decode("utf-8")
+    num_bytes = len(s.encode("utf-8"))
+    return (
+        (unicodedata.normalize("NFC", s) == s)
+        and (s not in _reserved_names)
+        and (num_bytes >= 0)
+        and ("/" not in s)
+        and (s[-1] != " ")
+        and (_isalnumMUTF8(s[0]) or (s[0] == "_"))
+        and all(_isalnumMUTF8(c) or c in _specialchars for c in s)
+    )
