@@ -53,34 +53,36 @@ from .times import _STANDARD_CALENDARS, cftime_to_nptime, infer_calendar_name
 
 
 def named(name, pattern):
-    return '(?P<' + name + '>' + pattern + ')'
+    return "(?P<" + name + ">" + pattern + ")"
 
 
 def optional(x):
-    return '(?:' + x + ')?'
+    return "(?:" + x + ")?"
 
 
 def trailing_optional(xs):
     if not xs:
-        return ''
+        return ""
     return xs[0] + optional(trailing_optional(xs[1:]))
 
 
-def build_pattern(date_sep=r'\-', datetime_sep=r'T', time_sep=r'\:'):
-    pieces = [(None, 'year', r'\d{4}'),
-              (date_sep, 'month', r'\d{2}'),
-              (date_sep, 'day', r'\d{2}'),
-              (datetime_sep, 'hour', r'\d{2}'),
-              (time_sep, 'minute', r'\d{2}'),
-              (time_sep, 'second', r'\d{2}')]
+def build_pattern(date_sep=r"\-", datetime_sep=r"T", time_sep=r"\:"):
+    pieces = [
+        (None, "year", r"\d{4}"),
+        (date_sep, "month", r"\d{2}"),
+        (date_sep, "day", r"\d{2}"),
+        (datetime_sep, "hour", r"\d{2}"),
+        (time_sep, "minute", r"\d{2}"),
+        (time_sep, "second", r"\d{2}"),
+    ]
     pattern_list = []
     for sep, name, sub_pattern in pieces:
-        pattern_list.append((sep if sep else '') + named(name, sub_pattern))
+        pattern_list.append((sep if sep else "") + named(name, sub_pattern))
         # TODO: allow timezone offsets?
-    return '^' + trailing_optional(pattern_list) + '$'
+    return "^" + trailing_optional(pattern_list) + "$"
 
 
-_BASIC_PATTERN = build_pattern(date_sep='', time_sep='')
+_BASIC_PATTERN = build_pattern(date_sep="", time_sep="")
 _EXTENDED_PATTERN = build_pattern()
 _PATTERNS = [_BASIC_PATTERN, _EXTENDED_PATTERN]
 
@@ -90,7 +92,7 @@ def parse_iso8601(datetime_string):
         match = re.match(pattern, datetime_string)
         if match:
             return match.groupdict()
-    raise ValueError('no ISO-8601 match for string: %s' % datetime_string)
+    raise ValueError("no ISO-8601 match for string: %s" % datetime_string)
 
 
 def _parse_iso8601_with_reso(date_type, timestr):
@@ -98,7 +100,7 @@ def _parse_iso8601_with_reso(date_type, timestr):
     result = parse_iso8601(timestr)
     replace = {}
 
-    for attr in ['year', 'month', 'day', 'hour', 'minute', 'second']:
+    for attr in ["year", "month", "day", "hour", "minute", "second"]:
         value = result.get(attr, None)
         if value is not None:
             # Note ISO8601 conventions allow for fractional seconds.
@@ -110,7 +112,7 @@ def _parse_iso8601_with_reso(date_type, timestr):
     # the returned date object in versions of cftime between 1.0.2 and
     # 1.0.3.4.  It can be removed for versions of cftime greater than
     # 1.0.3.4.
-    replace['dayofwk'] = -1
+    replace["dayofwk"] = -1
     return default.replace(**replace), resolution
 
 
@@ -120,29 +122,39 @@ def _parsed_string_to_bounds(date_type, resolution, parsed):
     for use with non-standard calendars and cftime.datetime
     objects.
     """
-    if resolution == 'year':
-        return (date_type(parsed.year, 1, 1),
-                date_type(parsed.year + 1, 1, 1) - timedelta(microseconds=1))
-    elif resolution == 'month':
+    if resolution == "year":
+        return (
+            date_type(parsed.year, 1, 1),
+            date_type(parsed.year + 1, 1, 1) - timedelta(microseconds=1),
+        )
+    elif resolution == "month":
         if parsed.month == 12:
             end = date_type(parsed.year + 1, 1, 1) - timedelta(microseconds=1)
         else:
-            end = (date_type(parsed.year, parsed.month + 1, 1) -
-                   timedelta(microseconds=1))
+            end = date_type(parsed.year, parsed.month + 1, 1) - timedelta(
+                microseconds=1
+            )
         return date_type(parsed.year, parsed.month, 1), end
-    elif resolution == 'day':
+    elif resolution == "day":
         start = date_type(parsed.year, parsed.month, parsed.day)
         return start, start + timedelta(days=1, microseconds=-1)
-    elif resolution == 'hour':
+    elif resolution == "hour":
         start = date_type(parsed.year, parsed.month, parsed.day, parsed.hour)
         return start, start + timedelta(hours=1, microseconds=-1)
-    elif resolution == 'minute':
-        start = date_type(parsed.year, parsed.month, parsed.day, parsed.hour,
-                          parsed.minute)
+    elif resolution == "minute":
+        start = date_type(
+            parsed.year, parsed.month, parsed.day, parsed.hour, parsed.minute
+        )
         return start, start + timedelta(minutes=1, microseconds=-1)
-    elif resolution == 'second':
-        start = date_type(parsed.year, parsed.month, parsed.day, parsed.hour,
-                          parsed.minute, parsed.second)
+    elif resolution == "second":
+        start = date_type(
+            parsed.year,
+            parsed.month,
+            parsed.day,
+            parsed.hour,
+            parsed.minute,
+            parsed.second,
+        )
         return start, start + timedelta(seconds=1, microseconds=-1)
     else:
         raise KeyError
@@ -153,7 +165,7 @@ def get_date_field(datetimes, field):
     return np.array([getattr(date, field) for date in datetimes])
 
 
-def _field_accessor(name, docstring=None, min_cftime_version='0.0'):
+def _field_accessor(name, docstring=None, min_cftime_version="0.0"):
     """Adapted from pandas.tseries.index._field_accessor"""
 
     def f(self, min_cftime_version=min_cftime_version):
@@ -164,10 +176,11 @@ def _field_accessor(name, docstring=None, min_cftime_version='0.0'):
         if LooseVersion(version) >= LooseVersion(min_cftime_version):
             return get_date_field(self._data, name)
         else:
-            raise ImportError('The {!r} accessor requires a minimum '
-                              'version of cftime of {}. Found an '
-                              'installed version of {}.'.format(
-                                  name, min_cftime_version, version))
+            raise ImportError(
+                "The {!r} accessor requires a minimum "
+                "version of cftime of {}. Found an "
+                "installed version of {}.".format(name, min_cftime_version, version)
+            )
 
     f.__name__ = name
     f.__doc__ = docstring
@@ -189,12 +202,14 @@ def assert_all_valid_date_type(data):
         date_type = type(sample)
         if not isinstance(sample, cftime.datetime):
             raise TypeError(
-                'CFTimeIndex requires cftime.datetime '
-                'objects. Got object of {}.'.format(date_type))
+                "CFTimeIndex requires cftime.datetime "
+                "objects. Got object of {}.".format(date_type)
+            )
         if not all(isinstance(value, date_type) for value in data):
             raise TypeError(
-                'CFTimeIndex requires using datetime '
-                'objects of all the same type.  Got\n{}.'.format(data))
+                "CFTimeIndex requires using datetime "
+                "objects of all the same type.  Got\n{}.".format(data)
+            )
 
 
 class CFTimeIndex(pd.Index):
@@ -213,28 +228,27 @@ class CFTimeIndex(pd.Index):
     --------
     cftime_range
     """
-    year = _field_accessor('year', 'The year of the datetime')
-    month = _field_accessor('month', 'The month of the datetime')
-    day = _field_accessor('day', 'The days of the datetime')
-    hour = _field_accessor('hour', 'The hours of the datetime')
-    minute = _field_accessor('minute', 'The minutes of the datetime')
-    second = _field_accessor('second', 'The seconds of the datetime')
-    microsecond = _field_accessor('microsecond',
-                                  'The microseconds of the datetime')
-    dayofyear = _field_accessor('dayofyr',
-                                'The ordinal day of year of the datetime',
-                                '1.0.2.1')
-    dayofweek = _field_accessor('dayofwk', 'The day of week of the datetime',
-                                '1.0.2.1')
+
+    year = _field_accessor("year", "The year of the datetime")
+    month = _field_accessor("month", "The month of the datetime")
+    day = _field_accessor("day", "The days of the datetime")
+    hour = _field_accessor("hour", "The hours of the datetime")
+    minute = _field_accessor("minute", "The minutes of the datetime")
+    second = _field_accessor("second", "The seconds of the datetime")
+    microsecond = _field_accessor("microsecond", "The microseconds of the datetime")
+    dayofyear = _field_accessor(
+        "dayofyr", "The ordinal day of year of the datetime", "1.0.2.1"
+    )
+    dayofweek = _field_accessor("dayofwk", "The day of week of the datetime", "1.0.2.1")
     date_type = property(get_date_type)
 
     def __new__(cls, data, name=None):
         assert_all_valid_date_type(data)
-        if name is None and hasattr(data, 'name'):
+        if name is None and hasattr(data, "name"):
             name = data.name
 
         result = object.__new__(cls)
-        result._data = np.array(data, dtype='O')
+        result._data = np.array(data, dtype="O")
         result.name = name
         return result
 
@@ -280,20 +294,21 @@ class CFTimeIndex(pd.Index):
         Coordinates:
           * time     (time) datetime64[ns] 2001-01-01T01:00:00
         """
-        start, end = _parsed_string_to_bounds(self.date_type, resolution,
-                                              parsed)
+        start, end = _parsed_string_to_bounds(self.date_type, resolution, parsed)
 
         times = self._data
 
         if self.is_monotonic:
-            if (len(times) and ((start < times[0] and end < times[0]) or
-                                (start > times[-1] and end > times[-1]))):
+            if len(times) and (
+                (start < times[0] and end < times[0])
+                or (start > times[-1] and end > times[-1])
+            ):
                 # we are out of range
                 raise KeyError
 
             # a monotonic (sorted) series can be sliced
-            left = times.searchsorted(start, side='left')
-            right = times.searchsorted(end, side='right')
+            left = times.searchsorted(start, side="left")
+            right = times.searchsorted(end, side="right")
             return slice(left, right)
 
         lhs_mask = times >= start
@@ -314,20 +329,17 @@ class CFTimeIndex(pd.Index):
         if isinstance(key, str):
             return self._get_string_slice(key)
         else:
-            return pd.Index.get_loc(self, key, method=method,
-                                    tolerance=tolerance)
+            return pd.Index.get_loc(self, key, method=method, tolerance=tolerance)
 
     def _maybe_cast_slice_bound(self, label, side, kind):
         """Adapted from
         pandas.tseries.index.DatetimeIndex._maybe_cast_slice_bound"""
         if isinstance(label, str):
-            parsed, resolution = _parse_iso8601_with_reso(self.date_type,
-                                                          label)
-            start, end = _parsed_string_to_bounds(self.date_type, resolution,
-                                                  parsed)
+            parsed, resolution = _parse_iso8601_with_reso(self.date_type, label)
+            start, end = _parsed_string_to_bounds(self.date_type, resolution, parsed)
             if self.is_monotonic_decreasing and len(self) > 1:
-                return end if side == 'left' else start
-            return start if side == 'left' else end
+                return end if side == "left" else start
+            return start if side == "left" else end
         else:
             return label
 
@@ -338,8 +350,7 @@ class CFTimeIndex(pd.Index):
         if np.asarray(key).dtype == np.dtype(bool):
             return series.iloc[key]
         elif isinstance(key, slice):
-            return series.iloc[self.slice_indexer(
-                key.start, key.stop, key.step)]
+            return series.iloc[self.slice_indexer(key.start, key.stop, key.step)]
         else:
             return series.iloc[self.get_loc(key)]
 
@@ -348,8 +359,11 @@ class CFTimeIndex(pd.Index):
         pandas.tseries.base.DatetimeIndexOpsMixin.__contains__"""
         try:
             result = self.get_loc(key)
-            return (is_scalar(result) or type(result) == slice or
-                    (isinstance(result, np.ndarray) and result.size))
+            return (
+                is_scalar(result)
+                or type(result) == slice
+                or (isinstance(result, np.ndarray) and result.size)
+            )
         except (KeyError, TypeError, ValueError):
             return False
 
@@ -397,7 +411,8 @@ class CFTimeIndex(pd.Index):
         else:
             raise TypeError(
                 "'freq' must be of type "
-                "str or datetime.timedelta, got {}.".format(freq))
+                "str or datetime.timedelta, got {}.".format(freq)
+            )
 
     def __add__(self, other):
         if isinstance(other, pd.TimedeltaIndex):
@@ -411,6 +426,7 @@ class CFTimeIndex(pd.Index):
 
     def __sub__(self, other):
         import cftime
+
         if isinstance(other, (CFTimeIndex, cftime.datetime)):
             return pd.TimedeltaIndex(np.array(self) - np.array(other))
         elif isinstance(other, pd.TimedeltaIndex):
@@ -464,16 +480,19 @@ class CFTimeIndex(pd.Index):
         CFTimeIndex([2000-01-01 00:00:00, 2000-01-02 00:00:00], dtype='object')
         >>> times.to_datetimeindex()
         DatetimeIndex(['2000-01-01', '2000-01-02'], dtype='datetime64[ns]', freq=None)
-        """  # noqa: E501
+        """
         nptimes = cftime_to_nptime(self)
         calendar = infer_calendar_name(self)
         if calendar not in _STANDARD_CALENDARS and not unsafe:
             warnings.warn(
-                'Converting a CFTimeIndex with dates from a non-standard '
-                'calendar, {!r}, to a pandas.DatetimeIndex, which uses dates '
-                'from the standard calendar.  This may lead to subtle errors '
-                'in operations that depend on the length of time between '
-                'dates.'.format(calendar), RuntimeWarning, stacklevel=2)
+                "Converting a CFTimeIndex with dates from a non-standard "
+                "calendar, {!r}, to a pandas.DatetimeIndex, which uses dates "
+                "from the standard calendar.  This may lead to subtle errors "
+                "in operations that depend on the length of time between "
+                "dates.".format(calendar),
+                RuntimeWarning,
+                stacklevel=2,
+            )
         return pd.DatetimeIndex(nptimes)
 
     def strftime(self, date_format):
@@ -528,5 +547,6 @@ def _parse_array_of_cftime_strings(strings, date_type):
     -------
     np.array
     """
-    return np.array([_parse_iso8601_without_reso(date_type, s)
-                     for s in strings.ravel()]).reshape(strings.shape)
+    return np.array(
+        [_parse_iso8601_without_reso(date_type, s) for s in strings.ravel()]
+    ).reshape(strings.shape)
