@@ -1,9 +1,8 @@
 import functools
 import operator
-import warnings
 from collections import OrderedDict, defaultdict
 from contextlib import suppress
-from typing import Any, Mapping, Optional, Tuple
+from typing import TYPE_CHECKING, Any, Dict, Hashable, Mapping, Optional, Tuple, Union
 
 import numpy as np
 import pandas as pd
@@ -13,26 +12,60 @@ from .indexing import get_indexer_nd
 from .utils import is_dict_like, is_full_slice
 from .variable import IndexVariable, Variable
 
+if TYPE_CHECKING:
+    from .dataarray import DataArray  # noqa: F401
+    from .dataset import Dataset  # noqa: F401
+
 
 def _get_joiner(join):
-    if join == 'outer':
+    if join == "outer":
         return functools.partial(functools.reduce, operator.or_)
-    elif join == 'inner':
+    elif join == "inner":
         return functools.partial(functools.reduce, operator.and_)
-    elif join == 'left':
+    elif join == "left":
         return operator.itemgetter(0)
-    elif join == 'right':
+    elif join == "right":
         return operator.itemgetter(-1)
-    elif join == 'exact':
+    elif join == "exact":
         # We cannot return a function to "align" in this case, because it needs
         # access to the dimension name to give a good error message.
         return None
+    elif join == "override":
+        # We rewrite all indexes and then use join='left'
+        return operator.itemgetter(0)
     else:
-        raise ValueError('invalid value for join: %s' % join)
+        raise ValueError("invalid value for join: %s" % join)
 
 
-def align(*objects, join='inner', copy=True, indexes=None, exclude=frozenset(),
-          fill_value=dtypes.NA):
+def _override_indexes(objects, all_indexes, exclude):
+    for dim, dim_indexes in all_indexes.items():
+        if dim not in exclude:
+            lengths = {index.size for index in dim_indexes}
+            if len(lengths) != 1:
+                raise ValueError(
+                    "Indexes along dimension %r don't have the same length."
+                    " Cannot use join='override'." % dim
+                )
+
+    objects = list(objects)
+    for idx, obj in enumerate(objects[1:]):
+        new_indexes = dict()
+        for dim in obj.dims:
+            if dim not in exclude:
+                new_indexes[dim] = all_indexes[dim][0]
+        objects[idx + 1] = obj._overwrite_indexes(new_indexes)
+
+    return objects
+
+
+def align(
+    *objects,
+    join="inner",
+    copy=True,
+    indexes=None,
+    exclude=frozenset(),
+    fill_value=dtypes.NA
+):
     """
     Given any number of Dataset and/or DataArray objects, returns new
     objects with aligned indexes and dimension sizes.
@@ -47,7 +80,7 @@ def align(*objects, join='inner', copy=True, indexes=None, exclude=frozenset(),
     ----------
     *objects : Dataset or DataArray
         Objects to align.
-    join : {'outer', 'inner', 'left', 'right', 'exact'}, optional
+    join : {'outer', 'inner', 'left', 'right', 'exact', 'override'}, optional
         Method for joining the indexes of the passed objects along each
         dimension:
 
@@ -57,6 +90,9 @@ def align(*objects, join='inner', copy=True, indexes=None, exclude=frozenset(),
         - 'right': use indexes from the last object with each dimension
         - 'exact': instead of aligning, raise `ValueError` when indexes to be
           aligned are not equal
+        - 'override': if indexes are of same size, rewrite indexes to be
+          those of the first object with that dimension. Indexes for the same
+          dimension must have the same size in all objects.
     copy : bool, optional
         If ``copy=True``, data in the return values is always copied. If
         ``copy=False`` and reindexing is unnecessary, or can be performed with
@@ -101,6 +137,9 @@ def align(*objects, join='inner', copy=True, indexes=None, exclude=frozenset(),
                 else:
                     all_indexes[dim].append(index)
 
+    if join == "override":
+        objects = _override_indexes(list(objects), all_indexes, exclude)
+
     # We don't reindex over dimensions with all equal indexes for two reasons:
     # - It's faster for the usual case (already aligned objects).
     # - It ensures it's possible to do operations that don't require alignment
@@ -111,17 +150,23 @@ def align(*objects, join='inner', copy=True, indexes=None, exclude=frozenset(),
     for dim, matching_indexes in all_indexes.items():
         if dim in indexes:
             index = utils.safe_cast_to_index(indexes[dim])
-            if (any(not index.equals(other) for other in matching_indexes) or
-                    dim in unlabeled_dim_sizes):
+            if (
+                any(not index.equals(other) for other in matching_indexes)
+                or dim in unlabeled_dim_sizes
+            ):
                 joined_indexes[dim] = index
         else:
-            if (any(not matching_indexes[0].equals(other)
-                    for other in matching_indexes[1:]) or
-                    dim in unlabeled_dim_sizes):
-                if join == 'exact':
+            if (
+                any(
+                    not matching_indexes[0].equals(other)
+                    for other in matching_indexes[1:]
+                )
+                or dim in unlabeled_dim_sizes
+            ):
+                if join == "exact":
                     raise ValueError(
-                        'indexes along dimension {!r} are not equal'
-                        .format(dim))
+                        "indexes along dimension {!r} are not equal".format(dim)
+                    )
                 index = joiner(matching_indexes)
                 joined_indexes[dim] = index
             else:
@@ -132,45 +177,51 @@ def align(*objects, join='inner', copy=True, indexes=None, exclude=frozenset(),
             labeled_size = index.size
             if len(unlabeled_sizes | {labeled_size}) > 1:
                 raise ValueError(
-                    'arguments without labels along dimension %r cannot be '
-                    'aligned because they have different dimension size(s) %r '
-                    'than the size of the aligned dimension labels: %r'
-                    % (dim, unlabeled_sizes, labeled_size))
+                    "arguments without labels along dimension %r cannot be "
+                    "aligned because they have different dimension size(s) %r "
+                    "than the size of the aligned dimension labels: %r"
+                    % (dim, unlabeled_sizes, labeled_size)
+                )
 
     for dim in unlabeled_dim_sizes:
         if dim not in all_indexes:
             sizes = unlabeled_dim_sizes[dim]
             if len(sizes) > 1:
                 raise ValueError(
-                    'arguments without labels along dimension %r cannot be '
-                    'aligned because they have different dimension sizes: %r'
-                    % (dim, sizes))
+                    "arguments without labels along dimension %r cannot be "
+                    "aligned because they have different dimension sizes: %r"
+                    % (dim, sizes)
+                )
 
     result = []
     for obj in objects:
-        valid_indexers = {k: v for k, v in joined_indexes.items()
-                          if k in obj.dims}
+        valid_indexers = {k: v for k, v in joined_indexes.items() if k in obj.dims}
         if not valid_indexers:
             # fast path for no reindexing necessary
             new_obj = obj.copy(deep=copy)
         else:
-            new_obj = obj.reindex(copy=copy, fill_value=fill_value,
-                                  **valid_indexers)
+            new_obj = obj.reindex(copy=copy, fill_value=fill_value, **valid_indexers)
         new_obj.encoding = obj.encoding
         result.append(new_obj)
 
     return tuple(result)
 
 
-def deep_align(objects, join='inner', copy=True, indexes=None,
-               exclude=frozenset(), raise_on_invalid=True,
-               fill_value=dtypes.NA):
+def deep_align(
+    objects,
+    join="inner",
+    copy=True,
+    indexes=None,
+    exclude=frozenset(),
+    raise_on_invalid=True,
+    fill_value=dtypes.NA,
+):
     """Align objects for merging, recursing into dictionary values.
 
     This function is not public API.
     """
-    from .dataarray import DataArray
-    from .dataset import Dataset
+    from .dataarray import DataArray  # noqa: F811
+    from .dataset import Dataset  # noqa: F811
 
     if indexes is None:
         indexes = {}
@@ -201,14 +252,21 @@ def deep_align(objects, join='inner', copy=True, indexes=None,
                     targets.append(v)
             out.append(OrderedDict(variables))
         elif raise_on_invalid:
-            raise ValueError('object to align is neither an xarray.Dataset, '
-                             'an xarray.DataArray nor a dictionary: %r'
-                             % variables)
+            raise ValueError(
+                "object to align is neither an xarray.Dataset, "
+                "an xarray.DataArray nor a dictionary: %r" % variables
+            )
         else:
             out.append(variables)
 
-    aligned = align(*targets, join=join, copy=copy, indexes=indexes,
-                    exclude=exclude, fill_value=fill_value)
+    aligned = align(
+        *targets,
+        join=join,
+        copy=copy,
+        indexes=indexes,
+        exclude=exclude,
+        fill_value=fill_value
+    )
 
     for position, key, aligned_obj in zip(positions, keys, aligned):
         if key is no_key:
@@ -222,7 +280,9 @@ def deep_align(objects, join='inner', copy=True, indexes=None,
     return out
 
 
-def reindex_like_indexers(target, other):
+def reindex_like_indexers(
+    target: Union["DataArray", "Dataset"], other: Union["DataArray", "Dataset"]
+) -> Dict[Hashable, pd.Index]:
     """Extract indexers to align target with other.
 
     Not public API.
@@ -236,7 +296,8 @@ def reindex_like_indexers(target, other):
 
     Returns
     -------
-    Dict[Any, pandas.Index] providing indexes for reindex keyword arguments.
+    Dict[Hashable, pandas.Index] providing indexes for reindex keyword
+    arguments.
 
     Raises
     ------
@@ -250,9 +311,11 @@ def reindex_like_indexers(target, other):
             other_size = other.sizes[dim]
             target_size = target.sizes[dim]
             if other_size != target_size:
-                raise ValueError('different size for unlabeled '
-                                 'dimension on argument %r: %r vs %r'
-                                 % (dim, other_size, target_size))
+                raise ValueError(
+                    "different size for unlabeled "
+                    "dimension on argument %r: %r vs %r"
+                    % (dim, other_size, target_size)
+                )
     return indexers
 
 
@@ -265,7 +328,7 @@ def reindex_variables(
     tolerance: Any = None,
     copy: bool = True,
     fill_value: Optional[Any] = dtypes.NA,
-) -> 'Tuple[OrderedDict[Any, Variable], OrderedDict[Any, pd.Index]]':
+) -> "Tuple[OrderedDict[Any, Variable], OrderedDict[Any, pd.Index]]":
     """Conform a dictionary of aligned variables onto a new set of variables,
     filling in missing values with NaN.
 
@@ -310,7 +373,7 @@ def reindex_variables(
     new_indexes : OrderedDict
         Dict of indexes associated with the reindexed variables.
     """
-    from .dataarray import DataArray
+    from .dataarray import DataArray  # noqa: F811
 
     # create variables for the new dataset
     reindexed = OrderedDict()  # type: OrderedDict[Any, Variable]
@@ -323,12 +386,10 @@ def reindex_variables(
 
     for dim, indexer in indexers.items():
         if isinstance(indexer, DataArray) and indexer.dims != (dim,):
-            warnings.warn(
-                "Indexer has dimensions {0:s} that are different "
-                "from that to be indexed along {1:s}. "
-                "This will behave differently in the future.".format(
-                    str(indexer.dims), dim),
-                FutureWarning, stacklevel=3)
+            raise ValueError(
+                "Indexer has dimensions {:s} that are different "
+                "from that to be indexed along {:s}".format(str(indexer.dims), dim)
+            )
 
         target = new_indexes[dim] = utils.safe_cast_to_index(indexers[dim])
 
@@ -337,8 +398,9 @@ def reindex_variables(
 
             if not index.is_unique:
                 raise ValueError(
-                    'cannot reindex or align along dimension %r because the '
-                    'index has duplicate values' % dim)
+                    "cannot reindex or align along dimension %r because the "
+                    "index has duplicate values" % dim
+                )
 
             int_indexer = get_indexer_nd(index, target, method, tolerance)
 
@@ -364,16 +426,17 @@ def reindex_variables(
             new_size = indexers[dim].size
             if existing_size != new_size:
                 raise ValueError(
-                    'cannot reindex or align along dimension %r without an '
-                    'index because its size %r is different from the size of '
-                    'the new index %r' % (dim, existing_size, new_size))
+                    "cannot reindex or align along dimension %r without an "
+                    "index because its size %r is different from the size of "
+                    "the new index %r" % (dim, existing_size, new_size)
+                )
 
     for name, var in variables.items():
         if name not in indexers:
-            key = tuple(slice(None)
-                        if d in unchanged_dims
-                        else int_indexers.get(d, slice(None))
-                        for d in var.dims)
+            key = tuple(
+                slice(None) if d in unchanged_dims else int_indexers.get(d, slice(None))
+                for d in var.dims
+            )
             needs_masking = any(d in masked_dims for d in var.dims)
 
             if needs_masking:
@@ -391,6 +454,55 @@ def reindex_variables(
     return reindexed, new_indexes
 
 
+def _get_broadcast_dims_map_common_coords(args, exclude):
+
+    common_coords = OrderedDict()
+    dims_map = OrderedDict()
+    for arg in args:
+        for dim in arg.dims:
+            if dim not in common_coords and dim not in exclude:
+                dims_map[dim] = arg.sizes[dim]
+                if dim in arg.coords:
+                    common_coords[dim] = arg.coords[dim].variable
+
+    return dims_map, common_coords
+
+
+def _broadcast_helper(arg, exclude, dims_map, common_coords):
+
+    from .dataarray import DataArray  # noqa: F811
+    from .dataset import Dataset  # noqa: F811
+
+    def _set_dims(var):
+        # Add excluded dims to a copy of dims_map
+        var_dims_map = dims_map.copy()
+        for dim in exclude:
+            with suppress(ValueError):
+                # ignore dim not in var.dims
+                var_dims_map[dim] = var.shape[var.dims.index(dim)]
+
+        return var.set_dims(var_dims_map)
+
+    def _broadcast_array(array):
+        data = _set_dims(array.variable)
+        coords = OrderedDict(array.coords)
+        coords.update(common_coords)
+        return DataArray(data, coords, data.dims, name=array.name, attrs=array.attrs)
+
+    def _broadcast_dataset(ds):
+        data_vars = OrderedDict((k, _set_dims(ds.variables[k])) for k in ds.data_vars)
+        coords = OrderedDict(ds.coords)
+        coords.update(common_coords)
+        return Dataset(data_vars, coords, ds.attrs)
+
+    if isinstance(arg, DataArray):
+        return _broadcast_array(arg)
+    elif isinstance(arg, Dataset):
+        return _broadcast_dataset(arg)
+    else:
+        raise ValueError("all input must be Dataset or DataArray objects")
+
+
 def broadcast(*args, exclude=None):
     """Explicitly broadcast any number of DataArray or Dataset objects against
     one another.
@@ -463,61 +575,14 @@ def broadcast(*args, exclude=None):
         a        (x, y) int64 1 1 2 2 3 3
         b        (x, y) int64 5 6 5 6 5 6
     """
-    from .dataarray import DataArray
-    from .dataset import Dataset
 
     if exclude is None:
         exclude = set()
-    args = align(*args, join='outer', copy=False, exclude=exclude)
-
-    common_coords = OrderedDict()
-    dims_map = OrderedDict()
-    for arg in args:
-        for dim in arg.dims:
-            if dim not in common_coords and dim not in exclude:
-                dims_map[dim] = arg.sizes[dim]
-                if dim in arg.coords:
-                    common_coords[dim] = arg.coords[dim].variable
-
-    def _set_dims(var):
-        # Add excluded dims to a copy of dims_map
-        var_dims_map = dims_map.copy()
-        for dim in exclude:
-            with suppress(ValueError):
-                # ignore dim not in var.dims
-                var_dims_map[dim] = var.shape[var.dims.index(dim)]
-
-        return var.set_dims(var_dims_map)
-
-    def _broadcast_array(array):
-        data = _set_dims(array.variable)
-        coords = OrderedDict(array.coords)
-        coords.update(common_coords)
-        return DataArray(data, coords, data.dims, name=array.name,
-                         attrs=array.attrs)
-
-    def _broadcast_dataset(ds):
-        data_vars = OrderedDict(
-            (k, _set_dims(ds.variables[k]))
-            for k in ds.data_vars)
-        coords = OrderedDict(ds.coords)
-        coords.update(common_coords)
-        return Dataset(data_vars, coords, ds.attrs)
+    args = align(*args, join="outer", copy=False, exclude=exclude)
 
+    dims_map, common_coords = _get_broadcast_dims_map_common_coords(args, exclude)
     result = []
     for arg in args:
-        if isinstance(arg, DataArray):
-            result.append(_broadcast_array(arg))
-        elif isinstance(arg, Dataset):
-            result.append(_broadcast_dataset(arg))
-        else:
-            raise ValueError('all input must be Dataset or DataArray objects')
+        result.append(_broadcast_helper(arg, exclude, dims_map, common_coords))
 
     return tuple(result)
-
-
-def broadcast_arrays(*args):
-    import warnings
-    warnings.warn('xarray.broadcast_arrays is deprecated: use '
-                  'xarray.broadcast instead', DeprecationWarning, stacklevel=2)
-    return broadcast(*args)
