@@ -1,9 +1,20 @@
+import warnings
 from collections import OrderedDict
 from contextlib import suppress
 from textwrap import dedent
 from typing import (
-    Any, Callable, Hashable, Iterable, Iterator, List, Mapping, MutableMapping,
-    Optional, Tuple, TypeVar, Union)
+    Any,
+    Callable,
+    Hashable,
+    Iterable,
+    Iterator,
+    List,
+    Mapping,
+    MutableMapping,
+    Tuple,
+    TypeVar,
+    Union,
+)
 
 import numpy as np
 import pandas as pd
@@ -17,68 +28,83 @@ from .rolling_exp import RollingExp
 from .utils import Frozen, ReprObject, SortedKeysDict, either_dict_or_kwargs
 
 # Used as a sentinel value to indicate a all dimensions
-ALL_DIMS = ReprObject('<all-dims>')
+ALL_DIMS = ReprObject("<all-dims>")
 
 
-T = TypeVar('T')
+C = TypeVar("C")
+T = TypeVar("T")
 
 
 class ImplementsArrayReduce:
+    __slots__ = ()
+
     @classmethod
-    def _reduce_method(cls, func: Callable, include_skipna: bool,
-                       numeric_only: bool):
+    def _reduce_method(cls, func: Callable, include_skipna: bool, numeric_only: bool):
         if include_skipna:
-            def wrapped_func(self, dim=None, axis=None, skipna=None,
-                             **kwargs):
-                return self.reduce(func, dim, axis,
-                                   skipna=skipna, allow_lazy=True, **kwargs)
+
+            def wrapped_func(self, dim=None, axis=None, skipna=None, **kwargs):
+                return self.reduce(
+                    func, dim, axis, skipna=skipna, allow_lazy=True, **kwargs
+                )
+
         else:
-            def wrapped_func(self, dim=None, axis=None,  # type: ignore
-                             **kwargs):
-                return self.reduce(func, dim, axis,
-                                   allow_lazy=True, **kwargs)
+
+            def wrapped_func(self, dim=None, axis=None, **kwargs):  # type: ignore
+                return self.reduce(func, dim, axis, allow_lazy=True, **kwargs)
+
         return wrapped_func
 
-    _reduce_extra_args_docstring = dedent("""\
+    _reduce_extra_args_docstring = dedent(
+        """\
         dim : str or sequence of str, optional
             Dimension(s) over which to apply `{name}`.
         axis : int or sequence of int, optional
             Axis(es) over which to apply `{name}`. Only one of the 'dim'
             and 'axis' arguments can be supplied. If neither are supplied, then
-            `{name}` is calculated over axes.""")
+            `{name}` is calculated over axes."""
+    )
 
-    _cum_extra_args_docstring = dedent("""\
+    _cum_extra_args_docstring = dedent(
+        """\
         dim : str or sequence of str, optional
             Dimension over which to apply `{name}`.
         axis : int or sequence of int, optional
             Axis over which to apply `{name}`. Only one of the 'dim'
-            and 'axis' arguments can be supplied.""")
+            and 'axis' arguments can be supplied."""
+    )
 
 
 class ImplementsDatasetReduce:
+    __slots__ = ()
+
     @classmethod
-    def _reduce_method(cls, func: Callable, include_skipna: bool,
-                       numeric_only: bool):
+    def _reduce_method(cls, func: Callable, include_skipna: bool, numeric_only: bool):
         if include_skipna:
-            def wrapped_func(self, dim=None, skipna=None,
-                             **kwargs):
-                return self.reduce(func, dim, skipna=skipna,
-                                   numeric_only=numeric_only, allow_lazy=True,
-                                   **kwargs)
+
+            def wrapped_func(self, dim=None, skipna=None, **kwargs):
+                return self.reduce(
+                    func,
+                    dim,
+                    skipna=skipna,
+                    numeric_only=numeric_only,
+                    allow_lazy=True,
+                    **kwargs
+                )
+
         else:
+
             def wrapped_func(self, dim=None, **kwargs):  # type: ignore
-                return self.reduce(func, dim,
-                                   numeric_only=numeric_only, allow_lazy=True,
-                                   **kwargs)
+                return self.reduce(
+                    func, dim, numeric_only=numeric_only, allow_lazy=True, **kwargs
+                )
+
         return wrapped_func
 
-    _reduce_extra_args_docstring = \
-        """dim : str or sequence of str, optional
+    _reduce_extra_args_docstring = """dim : str or sequence of str, optional
             Dimension(s) over which to apply `{name}`.  By default `{name}` is
             applied over all dimensions."""
 
-    _cum_extra_args_docstring = \
-        """dim : str or sequence of str, optional
+    _cum_extra_args_docstring = """dim : str or sequence of str, optional
             Dimension over which to apply `{name}`.
         axis : int or sequence of int, optional
             Axis over which to apply `{name}`. Only one of the 'dim'
@@ -89,6 +115,8 @@ class AbstractArray(ImplementsArrayReduce):
     """Shared base class for DataArray and Variable.
     """
 
+    __slots__ = ()
+
     def __bool__(self: Any) -> bool:
         return bool(self.values)
 
@@ -101,7 +129,7 @@ class AbstractArray(ImplementsArrayReduce):
     def __complex__(self: Any) -> complex:
         return complex(self.values)
 
-    def __array__(self: Any, dtype: Optional[DTypeLike] = None) -> np.ndarray:
+    def __array__(self: Any, dtype: DTypeLike = None) -> np.ndarray:
         return np.asarray(self.values, dtype=dtype)
 
     def __repr__(self) -> str:
@@ -113,11 +141,12 @@ class AbstractArray(ImplementsArrayReduce):
 
     def __iter__(self: Any) -> Iterator[Any]:
         if self.ndim == 0:
-            raise TypeError('iteration over a 0-d array')
+            raise TypeError("iteration over a 0-d array")
         return self._iter()
 
-    def get_axis_num(self, dim: Union[Hashable, Iterable[Hashable]]
-                     ) -> Union[int, Tuple[int, ...]]:
+    def get_axis_num(
+        self, dim: Union[Hashable, Iterable[Hashable]]
+    ) -> Union[int, Tuple[int, ...]]:
         """Return axis number(s) corresponding to dimension(s) in this array.
 
         Parameters
@@ -139,8 +168,7 @@ class AbstractArray(ImplementsArrayReduce):
         try:
             return self.dims.index(dim)
         except ValueError:
-            raise ValueError("%r not found in array dimensions %r" %
-                             (dim, self.dims))
+            raise ValueError("%r not found in array dimensions %r" % (dim, self.dims))
 
     @property
     def sizes(self: Any) -> Mapping[Hashable, int]:
@@ -158,7 +186,26 @@ class AbstractArray(ImplementsArrayReduce):
 class AttrAccessMixin:
     """Mixin class that allows getting keys with attribute access
     """
-    _initialized = False
+
+    __slots__ = ()
+
+    def __init_subclass__(cls):
+        """Verify that all subclasses explicitly define ``__slots__``. If they don't,
+        raise error in the core xarray module and a FutureWarning in third-party
+        extensions.
+        This check is only triggered in Python 3.6+.
+        """
+        if not hasattr(object.__new__(cls), "__dict__"):
+            cls.__setattr__ = cls._setattr_slots
+        elif cls.__module__.startswith("xarray."):
+            raise AttributeError("%s must explicitly define __slots__" % cls.__name__)
+        else:
+            cls.__setattr__ = cls._setattr_dict
+            warnings.warn(
+                "xarray subclass %s should explicitly define __slots__" % cls.__name__,
+                FutureWarning,
+                stacklevel=2,
+            )
 
     @property
     def _attr_sources(self) -> List[Mapping[Hashable, Any]]:
@@ -173,37 +220,73 @@ class AttrAccessMixin:
         return []
 
     def __getattr__(self, name: str) -> Any:
-        if name != '__setstate__':
+        if name not in {"__dict__", "__setstate__"}:
             # this avoids an infinite loop when pickle looks for the
             # __setstate__ attribute before the xarray object is initialized
             for source in self._attr_sources:
                 with suppress(KeyError):
                     return source[name]
-        raise AttributeError("%r object has no attribute %r" %
-                             (type(self).__name__, name))
+        raise AttributeError(
+            "%r object has no attribute %r" % (type(self).__name__, name)
+        )
 
-    def __setattr__(self, name: str, value: Any) -> None:
-        if self._initialized:
-            try:
-                # Allow setting instance variables if they already exist
-                # (e.g., _attrs). We use __getattribute__ instead of hasattr
-                # to avoid key lookups with attribute-style access.
-                self.__getattribute__(name)
-            except AttributeError:
-                raise AttributeError(
-                    "cannot set attribute %r on a %r object. Use __setitem__ "
-                    "style assignment (e.g., `ds['name'] = ...`) instead to "
-                    "assign variables." % (name, type(self).__name__))
+    # This complicated three-method design boosts overall performance of simple
+    # operations - particularly DataArray methods that perform a _to_temp_dataset()
+    # round-trip - by a whopping 8% compared to a single method that checks
+    # hasattr(self, "__dict__") at runtime before every single assignment (like
+    # _setattr_py35 does). All of this is just temporary until the FutureWarning can be
+    # changed into a hard crash.
+    def _setattr_dict(self, name: str, value: Any) -> None:
+        """Deprecated third party subclass (see ``__init_subclass__`` above)
+        """
         object.__setattr__(self, name, value)
+        if name in self.__dict__:
+            # Custom, non-slotted attr, or improperly assigned variable?
+            warnings.warn(
+                "Setting attribute %r on a %r object. Explicitly define __slots__ "
+                "to suppress this warning for legitimate custom attributes and "
+                "raise an error when attempting variables assignments."
+                % (name, type(self).__name__),
+                FutureWarning,
+                stacklevel=2,
+            )
+
+    def _setattr_slots(self, name: str, value: Any) -> None:
+        """Objects with ``__slots__`` raise AttributeError if you try setting an
+        undeclared attribute. This is desirable, but the error message could use some
+        improvement.
+        """
+        try:
+            object.__setattr__(self, name, value)
+        except AttributeError as e:
+            # Don't accidentally shadow custom AttributeErrors, e.g.
+            # DataArray.dims.setter
+            if str(e) != "%r object has no attribute %r" % (type(self).__name__, name):
+                raise
+            raise AttributeError(
+                "cannot set attribute %r on a %r object. Use __setitem__ style"
+                "assignment (e.g., `ds['name'] = ...`) instead of assigning variables."
+                % (name, type(self).__name__)
+            ) from e
+
+    def _setattr_py35(self, name: str, value: Any) -> None:
+        if hasattr(self, "__dict__"):
+            return self._setattr_dict(name, value)
+        return self._setattr_slots(name, value)
+
+    # Overridden in Python >=3.6 by __init_subclass__
+    __setattr__ = _setattr_py35
 
     def __dir__(self) -> List[str]:
         """Provide method name lookup and completion. Only provide 'public'
         methods.
         """
-        extra_attrs = [item
-                       for sublist in self._attr_sources
-                       for item in sublist
-                       if isinstance(item, str)]
+        extra_attrs = [
+            item
+            for sublist in self._attr_sources
+            for item in sublist
+            if isinstance(item, str)
+        ]
         return sorted(set(dir(type(self)) + extra_attrs))
 
     def _ipython_key_completions_(self) -> List[str]:
@@ -211,21 +294,24 @@ class AttrAccessMixin:
         See http://ipython.readthedocs.io/en/stable/config/integrating.html#tab-completion
         For the details.
         """  # noqa
-        item_lists = [item
-                      for sublist in self._item_sources
-                      for item in sublist
-                      if isinstance(item, str)]
+        item_lists = [
+            item
+            for sublist in self._item_sources
+            for item in sublist
+            if isinstance(item, str)
+        ]
         return list(set(item_lists))
 
 
-def get_squeeze_dims(xarray_obj,
-                     dim: Union[Hashable, Iterable[Hashable], None] = None,
-                     axis: Union[int, Iterable[int], None] = None
-                     ) -> List[Hashable]:
+def get_squeeze_dims(
+    xarray_obj,
+    dim: Union[Hashable, Iterable[Hashable], None] = None,
+    axis: Union[int, Iterable[int], None] = None,
+) -> List[Hashable]:
     """Get a list of dimensions to squeeze out.
     """
     if dim is not None and axis is not None:
-        raise ValueError('cannot use both parameters `axis` and `dim`')
+        raise ValueError("cannot use both parameters `axis` and `dim`")
     if dim is None and axis is None:
         return [d for d, s in xarray_obj.sizes.items() if s == 1]
 
@@ -239,25 +325,31 @@ def get_squeeze_dims(xarray_obj,
             axis = [axis]
         axis = list(axis)
         if any(not isinstance(a, int) for a in axis):
-            raise TypeError(
-                'parameter `axis` must be int or iterable of int.')
+            raise TypeError("parameter `axis` must be int or iterable of int.")
         alldims = list(xarray_obj.sizes.keys())
         dim = [alldims[a] for a in axis]
 
     if any(xarray_obj.sizes[k] > 1 for k in dim):
-        raise ValueError('cannot select a dimension to squeeze out '
-                         'which has length greater than one')
+        raise ValueError(
+            "cannot select a dimension to squeeze out "
+            "which has length greater than one"
+        )
     return dim
 
 
 class DataWithCoords(SupportsArithmetic, AttrAccessMixin):
     """Shared base class for Dataset and DataArray."""
 
+    __slots__ = ()
+
     _rolling_exp_cls = RollingExp
 
-    def squeeze(self, dim: Union[Hashable, Iterable[Hashable], None] = None,
-                drop: bool = False,
-                axis: Union[int, Iterable[int], None] = None):
+    def squeeze(
+        self,
+        dim: Union[Hashable, Iterable[Hashable], None] = None,
+        drop: bool = False,
+        axis: Union[int, Iterable[int], None] = None,
+    ):
         """Return a new object with squeezed data.
 
         Parameters
@@ -297,9 +389,10 @@ class DataWithCoords(SupportsArithmetic, AttrAccessMixin):
             # need to ensure dtype=int64 in case range is empty on Python 2
             return pd.Index(range(self.sizes[key]), name=key, dtype=np.int64)
 
-    def _calc_assign_results(self, kwargs: Mapping[str, T]
-                             ) -> MutableMapping[str, T]:
-        results = SortedKeysDict()  # type: SortedKeysDict[str, T]
+    def _calc_assign_results(
+        self: C, kwargs: Mapping[Hashable, Union[T, Callable[[C], T]]]
+    ) -> MutableMapping[Hashable, T]:
+        results = SortedKeysDict()  # type: SortedKeysDict[Hashable, T]
         for k, v in kwargs.items():
             if callable(v):
                 results[k] = v(self)
@@ -307,7 +400,7 @@ class DataWithCoords(SupportsArithmetic, AttrAccessMixin):
                 results[k] = v
         return results
 
-    def assign_coords(self, **kwargs):
+    def assign_coords(self, coords=None, **coords_kwargs):
         """Assign new coordinates to this object.
 
         Returns a new object with all the original data in addition to the new
@@ -315,11 +408,15 @@ class DataWithCoords(SupportsArithmetic, AttrAccessMixin):
 
         Parameters
         ----------
-        kwargs : keyword, value pairs
-            keywords are the variables names. If the values are callable, they
-            are computed on this object and assigned to new coordinate
-            variables. If the values are not callable, (e.g. a DataArray,
-            scalar, or array), they are simply assigned.
+        coords : dict, optional
+            A dict with keys which are variables names. If the values are
+            callable, they are computed on this object and assigned to new
+            coordinate variables. If the values are not callable,
+            (e.g. a ``DataArray``, scalar, or array), they are simply assigned.
+
+        **coords_kwargs : keyword, value pairs, optional
+            The keyword arguments form of ``coords``.
+            One of ``coords`` or ``coords_kwargs`` must be provided.
 
         Returns
         -------
@@ -329,7 +426,6 @@ class DataWithCoords(SupportsArithmetic, AttrAccessMixin):
 
         Examples
         --------
-
         Convert longitude coordinates from 0-359 to -180-179:
 
         >>> da = xr.DataArray(np.random.rand(4),
@@ -346,10 +442,18 @@ class DataWithCoords(SupportsArithmetic, AttrAccessMixin):
         Coordinates:
           * lon      (lon) int64 -2 -1 0 1
 
+        The function also accepts dictionary arguments:
+
+        >>> da.assign_coords({'lon': (((da.lon + 180) % 360) - 180)})
+        <xarray.DataArray (lon: 4)>
+        array([0.28298 , 0.667347, 0.657938, 0.177683])
+        Coordinates:
+          * lon      (lon) int64 -2 -1 0 1
+
         Notes
         -----
-        Since ``kwargs`` is a dictionary, the order of your arguments may not
-        be preserved, and so the order of the new variables is not well
+        Since ``coords_kwargs`` is a dictionary, the order of your arguments may
+        not be preserved, and so the order of the new variables is not well
         defined. Assigning multiple variables within the same ``assign_coords``
         is possible, but you cannot reference other variables created within
         the same ``assign_coords`` call.
@@ -359,8 +463,9 @@ class DataWithCoords(SupportsArithmetic, AttrAccessMixin):
         Dataset.assign
         Dataset.swap_dims
         """
+        coords_kwargs = either_dict_or_kwargs(coords, coords_kwargs, "assign_coords")
         data = self.copy(deep=False)
-        results = self._calc_assign_results(kwargs)
+        results = self._calc_assign_results(coords_kwargs)
         data.coords.update(results)
         return data
 
@@ -387,8 +492,12 @@ class DataWithCoords(SupportsArithmetic, AttrAccessMixin):
         out.attrs.update(*args, **kwargs)
         return out
 
-    def pipe(self, func: Union[Callable[..., T], Tuple[Callable[..., T], str]],
-             *args, **kwargs) -> T:
+    def pipe(
+        self,
+        func: Union[Callable[..., T], Tuple[Callable[..., T], str]],
+        *args,
+        **kwargs
+    ) -> T:
         """
         Apply func(self, *args, **kwargs)
 
@@ -440,15 +549,15 @@ class DataWithCoords(SupportsArithmetic, AttrAccessMixin):
         if isinstance(func, tuple):
             func, target = func
             if target in kwargs:
-                raise ValueError('%s is both the pipe target and a keyword '
-                                 'argument' % target)
+                raise ValueError(
+                    "%s is both the pipe target and a keyword " "argument" % target
+                )
             kwargs[target] = self
             return func(*args, **kwargs)
         else:
             return func(self, *args, **kwargs)
 
-    def groupby(self, group, squeeze: bool = True,
-                restore_coord_dims: Optional[bool] = None):
+    def groupby(self, group, squeeze: bool = True, restore_coord_dims: bool = None):
         """Returns a GroupBy object for performing grouped operations.
 
         Parameters
@@ -495,13 +604,21 @@ class DataWithCoords(SupportsArithmetic, AttrAccessMixin):
         core.groupby.DataArrayGroupBy
         core.groupby.DatasetGroupBy
         """  # noqa
-        return self._groupby_cls(self, group, squeeze=squeeze,
-                                 restore_coord_dims=restore_coord_dims)
+        return self._groupby_cls(
+            self, group, squeeze=squeeze, restore_coord_dims=restore_coord_dims
+        )
 
-    def groupby_bins(self, group, bins, right: bool = True, labels=None,
-                     precision: int = 3, include_lowest: bool = False,
-                     squeeze: bool = True,
-                     restore_coord_dims: Optional[bool] = None):
+    def groupby_bins(
+        self,
+        group,
+        bins,
+        right: bool = True,
+        labels=None,
+        precision: int = 3,
+        include_lowest: bool = False,
+        squeeze: bool = True,
+        restore_coord_dims: bool = None,
+    ):
         """Returns a GroupBy object for performing grouped operations.
 
         Rather than using all unique values of `group`, the values are discretized
@@ -550,16 +667,27 @@ class DataWithCoords(SupportsArithmetic, AttrAccessMixin):
         ----------
         .. [1] http://pandas.pydata.org/pandas-docs/stable/generated/pandas.cut.html
         """  # noqa
-        return self._groupby_cls(self, group, squeeze=squeeze, bins=bins,
-                                 restore_coord_dims=restore_coord_dims,
-                                 cut_kwargs={'right': right, 'labels': labels,
-                                             'precision': precision,
-                                             'include_lowest':
-                                                 include_lowest})
+        return self._groupby_cls(
+            self,
+            group,
+            squeeze=squeeze,
+            bins=bins,
+            restore_coord_dims=restore_coord_dims,
+            cut_kwargs={
+                "right": right,
+                "labels": labels,
+                "precision": precision,
+                "include_lowest": include_lowest,
+            },
+        )
 
-    def rolling(self, dim: Optional[Mapping[Hashable, int]] = None,
-                min_periods: Optional[int] = None, center: bool = False,
-                **window_kwargs: int):
+    def rolling(
+        self,
+        dim: Mapping[Hashable, int] = None,
+        min_periods: int = None,
+        center: bool = False,
+        **window_kwargs: int
+    ):
         """
         Rolling window object.
 
@@ -615,14 +743,13 @@ class DataWithCoords(SupportsArithmetic, AttrAccessMixin):
         core.rolling.DataArrayRolling
         core.rolling.DatasetRolling
         """  # noqa
-        dim = either_dict_or_kwargs(dim, window_kwargs, 'rolling')
-        return self._rolling_cls(self, dim, min_periods=min_periods,
-                                 center=center)
+        dim = either_dict_or_kwargs(dim, window_kwargs, "rolling")
+        return self._rolling_cls(self, dim, min_periods=min_periods, center=center)
 
     def rolling_exp(
         self,
-        window: Optional[Mapping[Hashable, int]] = None,
-        window_type: str = 'span',
+        window: Mapping[Hashable, int] = None,
+        window_type: str = "span",
         **window_kwargs
     ):
         """
@@ -654,15 +781,18 @@ class DataWithCoords(SupportsArithmetic, AttrAccessMixin):
         --------
         core.rolling_exp.RollingExp
         """
-        window = either_dict_or_kwargs(window, window_kwargs, 'rolling_exp')
+        window = either_dict_or_kwargs(window, window_kwargs, "rolling_exp")
 
         return self._rolling_exp_cls(self, window, window_type)
 
-    def coarsen(self, dim: Optional[Mapping[Hashable, int]] = None,
-                boundary: str = 'exact',
-                side: Union[str, Mapping[Hashable, str]] = 'left',
-                coord_func: str = 'mean',
-                **window_kwargs: int):
+    def coarsen(
+        self,
+        dim: Mapping[Hashable, int] = None,
+        boundary: str = "exact",
+        side: Union[str, Mapping[Hashable, str]] = "left",
+        coord_func: str = "mean",
+        **window_kwargs: int
+    ):
         """
         Coarsen object.
 
@@ -716,17 +846,23 @@ class DataWithCoords(SupportsArithmetic, AttrAccessMixin):
         core.rolling.DataArrayCoarsen
         core.rolling.DatasetCoarsen
         """
-        dim = either_dict_or_kwargs(dim, window_kwargs, 'coarsen')
+        dim = either_dict_or_kwargs(dim, window_kwargs, "coarsen")
         return self._coarsen_cls(
-            self, dim, boundary=boundary, side=side,
-            coord_func=coord_func)
+            self, dim, boundary=boundary, side=side, coord_func=coord_func
+        )
 
-    def resample(self, indexer: Optional[Mapping[Hashable, str]] = None,
-                 skipna=None, closed: Optional[str] = None,
-                 label: Optional[str] = None,
-                 base: int = 0, keep_attrs: Optional[bool] = None,
-                 loffset=None, restore_coord_dims: Optional[bool] = None,
-                 **indexer_kwargs: str):
+    def resample(
+        self,
+        indexer: Mapping[Hashable, str] = None,
+        skipna=None,
+        closed: str = None,
+        label: str = None,
+        base: int = 0,
+        keep_attrs: bool = None,
+        loffset=None,
+        restore_coord_dims: bool = None,
+        **indexer_kwargs: str
+    ):
         """Returns a Resample object for performing resampling operations.
 
         Handles both downsampling and upsampling. If any intervals contain no
@@ -814,19 +950,20 @@ class DataWithCoords(SupportsArithmetic, AttrAccessMixin):
             keep_attrs = _get_keep_attrs(default=False)
 
         # note: the second argument (now 'skipna') use to be 'dim'
-        if ((skipna is not None and not isinstance(skipna, bool))
-                or ('how' in indexer_kwargs and 'how' not in self.dims)
-                or ('dim' in indexer_kwargs and 'dim' not in self.dims)):
+        if (
+            (skipna is not None and not isinstance(skipna, bool))
+            or ("how" in indexer_kwargs and "how" not in self.dims)
+            or ("dim" in indexer_kwargs and "dim" not in self.dims)
+        ):
             raise TypeError(
-                'resample() no longer supports the `how` or '
-                '`dim` arguments. Instead call methods on resample '
-                "objects, e.g., data.resample(time='1D').mean()")
-
-        indexer = either_dict_or_kwargs(indexer, indexer_kwargs, 'resample')
-        if len(indexer) != 1:
-            raise ValueError(
-                "Resampling only supported along single dimensions."
+                "resample() no longer supports the `how` or "
+                "`dim` arguments. Instead call methods on resample "
+                "objects, e.g., data.resample(time='1D').mean()"
             )
+
+        indexer = either_dict_or_kwargs(indexer, indexer_kwargs, "resample")
+        if len(indexer) != 1:
+            raise ValueError("Resampling only supported along single dimensions.")
         dim, freq = next(iter(indexer.items()))
 
         dim_name = dim
@@ -834,19 +971,28 @@ class DataWithCoords(SupportsArithmetic, AttrAccessMixin):
 
         if isinstance(self.indexes[dim_name], CFTimeIndex):
             from .resample_cftime import CFTimeGrouper
+
             grouper = CFTimeGrouper(freq, closed, label, base, loffset)
         else:
             # TODO: to_offset() call required for pandas==0.19.2
-            grouper = pd.Grouper(freq=freq, closed=closed, label=label,
-                                 base=base,
-                                 loffset=pd.tseries.frequencies.to_offset(
-                                     loffset))
-        group = DataArray(dim_coord, coords=dim_coord.coords,
-                          dims=dim_coord.dims, name=RESAMPLE_DIM)
-        resampler = self._resample_cls(self, group=group, dim=dim_name,
-                                       grouper=grouper,
-                                       resample_dim=RESAMPLE_DIM,
-                                       restore_coord_dims=restore_coord_dims)
+            grouper = pd.Grouper(
+                freq=freq,
+                closed=closed,
+                label=label,
+                base=base,
+                loffset=pd.tseries.frequencies.to_offset(loffset),
+            )
+        group = DataArray(
+            dim_coord, coords=dim_coord.coords, dims=dim_coord.dims, name=RESAMPLE_DIM
+        )
+        resampler = self._resample_cls(
+            self,
+            group=group,
+            dim=dim_name,
+            grouper=grouper,
+            resample_dim=RESAMPLE_DIM,
+            restore_coord_dims=restore_coord_dims,
+        )
 
         return resampler
 
@@ -912,18 +1058,20 @@ class DataWithCoords(SupportsArithmetic, AttrAccessMixin):
 
         if drop:
             if other is not dtypes.NA:
-                raise ValueError('cannot set `other` if drop=True')
+                raise ValueError("cannot set `other` if drop=True")
 
             if not isinstance(cond, (Dataset, DataArray)):
-                raise TypeError("cond argument is %r but must be a %r or %r" %
-                                (cond, Dataset, DataArray))
+                raise TypeError(
+                    "cond argument is %r but must be a %r or %r"
+                    % (cond, Dataset, DataArray)
+                )
 
             # align so we can use integer indexing
             self, cond = align(self, cond)
 
             # get cond with the minimal size needed for the Dataset
             if isinstance(cond, Dataset):
-                clipcond = cond.to_array().any('variable')
+                clipcond = cond.to_array().any("variable")
             else:
                 clipcond = cond
 
@@ -978,8 +1126,10 @@ class DataWithCoords(SupportsArithmetic, AttrAccessMixin):
 
         if isinstance(test_elements, Dataset):
             raise TypeError(
-                'isin() argument must be convertible to an array: {}'
-                .format(test_elements))
+                "isin() argument must be convertible to an array: {}".format(
+                    test_elements
+                )
+            )
         elif isinstance(test_elements, (Variable, DataArray)):
             # need to explicitly pull out data to support dask arrays as the
             # second argument
@@ -989,7 +1139,7 @@ class DataWithCoords(SupportsArithmetic, AttrAccessMixin):
             duck_array_ops.isin,
             self,
             kwargs=dict(test_elements=test_elements),
-            dask='allowed',
+            dask="allowed",
         )
 
     def __enter__(self: T) -> T:
@@ -1003,7 +1153,7 @@ class DataWithCoords(SupportsArithmetic, AttrAccessMixin):
         raise NotImplementedError
 
 
-def full_like(other, fill_value, dtype: Optional[DTypeLike] = None):
+def full_like(other, fill_value, dtype: DTypeLike = None):
     """Return a new object with the same shape and type as a given object.
 
     Parameters
@@ -1030,44 +1180,49 @@ def full_like(other, fill_value, dtype: Optional[DTypeLike] = None):
     if isinstance(other, Dataset):
         data_vars = OrderedDict(
             (k, _full_like_variable(v, fill_value, dtype))
-            for k, v in other.data_vars.items())
+            for k, v in other.data_vars.items()
+        )
         return Dataset(data_vars, coords=other.coords, attrs=other.attrs)
     elif isinstance(other, DataArray):
         return DataArray(
             _full_like_variable(other.variable, fill_value, dtype),
-            dims=other.dims, coords=other.coords, attrs=other.attrs,
-            name=other.name)
+            dims=other.dims,
+            coords=other.coords,
+            attrs=other.attrs,
+            name=other.name,
+        )
     elif isinstance(other, Variable):
         return _full_like_variable(other, fill_value, dtype)
     else:
         raise TypeError("Expected DataArray, Dataset, or Variable")
 
 
-def _full_like_variable(other, fill_value,
-                        dtype: Optional[DTypeLike] = None):
+def _full_like_variable(other, fill_value, dtype: DTypeLike = None):
     """Inner function of full_like, where other must be a variable
     """
     from .variable import Variable
 
     if isinstance(other.data, dask_array_type):
         import dask.array
+
         if dtype is None:
             dtype = other.dtype
-        data = dask.array.full(other.shape, fill_value, dtype=dtype,
-                               chunks=other.data.chunks)
+        data = dask.array.full(
+            other.shape, fill_value, dtype=dtype, chunks=other.data.chunks
+        )
     else:
         data = np.full_like(other, fill_value, dtype=dtype)
 
     return Variable(dims=other.dims, data=data, attrs=other.attrs)
 
 
-def zeros_like(other, dtype: Optional[DTypeLike] = None):
+def zeros_like(other, dtype: DTypeLike = None):
     """Shorthand for full_like(other, 0, dtype)
     """
     return full_like(other, 0, dtype)
 
 
-def ones_like(other, dtype: Optional[DTypeLike] = None):
+def ones_like(other, dtype: DTypeLike = None):
     """Shorthand for full_like(other, 1, dtype)
     """
     return full_like(other, 1, dtype)
@@ -1076,8 +1231,7 @@ def ones_like(other, dtype: Optional[DTypeLike] = None):
 def is_np_datetime_like(dtype: DTypeLike) -> bool:
     """Check if a dtype is a subclass of the numpy datetime types
     """
-    return (np.issubdtype(dtype, np.datetime64) or
-            np.issubdtype(dtype, np.timedelta64))
+    return np.issubdtype(dtype, np.datetime64) or np.issubdtype(dtype, np.timedelta64)
 
 
 def _contains_cftime_datetimes(array) -> bool:
@@ -1088,7 +1242,7 @@ def _contains_cftime_datetimes(array) -> bool:
     except ImportError:
         return False
     else:
-        if array.dtype == np.dtype('O') and array.size > 0:
+        if array.dtype == np.dtype("O") and array.size > 0:
             sample = array.ravel()[0]
             if isinstance(sample, dask_array_type):
                 sample = sample.compute()
