@@ -1,33 +1,55 @@
 from collections import OrderedDict
-from typing import (Any, Dict, Hashable, List, Mapping, Optional, Sequence,
-                    Set, Tuple, Union)
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    Dict,
+    Hashable,
+    Iterable,
+    List,
+    Mapping,
+    MutableMapping,
+    Sequence,
+    Set,
+    Tuple,
+    Union,
+)
 
 import pandas as pd
 
 from . import dtypes, pdcompat
 from .alignment import deep_align
-from .pycompat import TYPE_CHECKING
 from .utils import Frozen
-from .variable import (
-    Variable, as_variable, assert_unique_multiindex_level_names)
+from .variable import Variable, as_variable, assert_unique_multiindex_level_names
 
 if TYPE_CHECKING:
     from .dataarray import DataArray
     from .dataset import Dataset
 
+    DatasetLikeValue = Union[
+        DataArray, Variable, Tuple[Hashable, Any], Tuple[Sequence[Hashable], Any]
+    ]
+    DatasetLike = Union[Dataset, Mapping[Hashable, DatasetLikeValue]]
+    """Any object type that can be used on the rhs of Dataset.update,
+    Dataset.merge, etc.
+    """
+    MutableDatasetLike = Union[Dataset, MutableMapping[Hashable, DatasetLikeValue]]
+
 
 PANDAS_TYPES = (pd.Series, pd.DataFrame, pdcompat.Panel)
 
-_VALID_COMPAT = Frozen({'identical': 0,
-                        'equals': 1,
-                        'broadcast_equals': 2,
-                        'minimal': 3,
-                        'no_conflicts': 4})
+_VALID_COMPAT = Frozen(
+    {
+        "identical": 0,
+        "equals": 1,
+        "broadcast_equals": 2,
+        "minimal": 3,
+        "no_conflicts": 4,
+        "override": 5,
+    }
+)
 
 
-def broadcast_dimension_size(
-    variables: List[Variable],
-) -> 'OrderedDict[Any, int]':
+def broadcast_dimension_size(variables: List[Variable],) -> "OrderedDict[Any, int]":
     """Extract dimension sizes from a dictionary of variables.
 
     Raises ValueError if any dimensions have different sizes.
@@ -36,7 +58,7 @@ def broadcast_dimension_size(
     for var in variables:
         for dim, size in zip(var.dims, var.shape):
             if dim in dims and size != dims[dim]:
-                raise ValueError('index %r not aligned' % dim)
+                raise ValueError("index %r not aligned" % dim)
             dims[dim] = size
     return dims
 
@@ -44,12 +66,13 @@ def broadcast_dimension_size(
 class MergeError(ValueError):
     """Error class for merge failures due to incompatible arguments.
     """
+
     # inherits from ValueError for backward compatibility
     # TODO: move this to an xarray.exceptions module?
 
 
-def unique_variable(name, variables, compat='broadcast_equals'):
-    # type: (Any, List[Variable], str) -> Variable
+def unique_variable(name, variables, compat="broadcast_equals", equals=None):
+    # type: (Any, List[Variable], str, bool) -> Variable
     """Return the unique variable from a list of variables or raise MergeError.
 
     Parameters
@@ -59,8 +82,10 @@ def unique_variable(name, variables, compat='broadcast_equals'):
     variables : list of xarray.Variable
         List of Variable objects, all of which go by the same name in different
         inputs.
-    compat : {'identical', 'equals', 'broadcast_equals', 'no_conflicts'}, optional
+    compat : {'identical', 'equals', 'broadcast_equals', 'no_conflicts', 'override'}, optional
         Type of equality check to use.
+    equals: None or bool,
+        corresponding to result of compat test
 
     Returns
     -------
@@ -71,37 +96,45 @@ def unique_variable(name, variables, compat='broadcast_equals'):
     MergeError: if any of the variables are not equal.
     """  # noqa
     out = variables[0]
-    if len(variables) > 1:
-        combine_method = None
 
-        if compat == 'minimal':
-            compat = 'broadcast_equals'
+    if len(variables) == 1 or compat == "override":
+        return out
 
-        if compat == 'broadcast_equals':
-            dim_lengths = broadcast_dimension_size(variables)
-            out = out.set_dims(dim_lengths)
+    combine_method = None
 
-        if compat == 'no_conflicts':
-            combine_method = 'fillna'
+    if compat == "minimal":
+        compat = "broadcast_equals"
 
+    if compat == "broadcast_equals":
+        dim_lengths = broadcast_dimension_size(variables)
+        out = out.set_dims(dim_lengths)
+
+    if compat == "no_conflicts":
+        combine_method = "fillna"
+
+    if equals is None:
+        out = out.compute()
         for var in variables[1:]:
-            if not getattr(out, compat)(var):
-                raise MergeError('conflicting values for variable %r on '
-                                 'objects to be combined:\n'
-                                 'first value: %r\nsecond value: %r'
-                                 % (name, out, var))
-            if combine_method:
-                # TODO: add preservation of attrs into fillna
-                out = getattr(out, combine_method)(var)
-                out.attrs = var.attrs
+            equals = getattr(out, compat)(var)
+            if not equals:
+                break
+
+    if not equals:
+        raise MergeError(
+            "conflicting values for variable %r on objects to be combined. You can skip this check by specifying compat='override'."
+            % (name)
+        )
+
+    if combine_method:
+        for var in variables[1:]:
+            out = getattr(out, combine_method)(var)
 
     return out
 
 
 def _assert_compat_valid(compat):
     if compat not in _VALID_COMPAT:
-        raise ValueError("compat=%r invalid: must be %s"
-                         % (compat, set(_VALID_COMPAT)))
+        raise ValueError("compat=%r invalid: must be %s" % (compat, set(_VALID_COMPAT)))
 
 
 class OrderedDefaultDict(OrderedDict):
@@ -117,11 +150,10 @@ class OrderedDefaultDict(OrderedDict):
 
 
 def merge_variables(
-        list_of_variables_dicts,  # type: List[Mapping[Any, Variable]]
-        priority_vars=None,       # type: Optional[Mapping[Any, Variable]]
-        compat='minimal',         # type: str
-):
-    # type: (...) -> OrderedDict[Any, Variable]
+    list_of_variables_dicts: List[Mapping[Any, Variable]],
+    priority_vars: Mapping[Any, Variable] = None,
+    compat: str = "minimal",
+) -> "OrderedDict[Any, Variable]":
     """Merge dicts of variables, while resolving conflicts appropriately.
 
     Parameters
@@ -131,7 +163,7 @@ def merge_variables(
     priority_vars : mapping with Variable or None values, optional
         If provided, variables are always taken from this dict in preference to
         the input variable dictionaries, without checking for conflicts.
-    compat : {'identical', 'equals', 'broadcast_equals', 'minimal', 'no_conflicts'}, optional
+    compat : {'identical', 'equals', 'broadcast_equals', 'minimal', 'no_conflicts', 'override'}, optional
         Type of equality check to use when checking for conflicts.
 
     Returns
@@ -144,7 +176,7 @@ def merge_variables(
         priority_vars = {}
 
     _assert_compat_valid(compat)
-    dim_compat = min(compat, 'equals', key=_VALID_COMPAT.get)
+    dim_compat = min(compat, "equals", key=_VALID_COMPAT.get)
 
     lookup = OrderedDefaultDict(list)
     for variables in list_of_variables_dicts:
@@ -171,7 +203,7 @@ def merge_variables(
                 try:
                     merged[name] = unique_variable(name, var_list, compat)
                 except MergeError:
-                    if compat != 'minimal':
+                    if compat != "minimal":
                         # we need more than "minimal" compatibility (for which
                         # we drop conflicting coordinates)
                         raise
@@ -180,8 +212,8 @@ def merge_variables(
 
 
 def expand_variable_dicts(
-    list_of_variable_dicts: 'List[Union[Dataset, OrderedDict]]',
-) -> 'List[Mapping[Any, Variable]]':
+    list_of_variable_dicts: "List[Union[Dataset, OrderedDict]]",
+) -> "List[Mapping[Any, Variable]]":
     """Given a list of dicts with xarray object values, expand the values.
 
     Parameters
@@ -229,8 +261,9 @@ def expand_variable_dicts(
     return var_dicts
 
 
-def determine_coords(list_of_variable_dicts):
-    # type: (List[Dict]) -> Tuple[Set, Set]
+def determine_coords(
+    list_of_variable_dicts: Iterable["DatasetLike"]
+) -> Tuple[Set[Hashable], Set[Hashable]]:
     """Given a list of dicts with xarray object values, identify coordinates.
 
     Parameters
@@ -266,7 +299,7 @@ def determine_coords(list_of_variable_dicts):
     return coord_names, noncoord_names
 
 
-def coerce_pandas_values(objects):
+def coerce_pandas_values(objects: Iterable["DatasetLike"]) -> List["DatasetLike"]:
     """Convert pandas values found in a list of labeled objects.
 
     Parameters
@@ -286,7 +319,7 @@ def coerce_pandas_values(objects):
     out = []
     for obj in objects:
         if isinstance(obj, Dataset):
-            variables = obj
+            variables = obj  # type: DatasetLike
         else:
             variables = OrderedDict()
             if isinstance(obj, PANDAS_TYPES):
@@ -310,7 +343,7 @@ def merge_coords_for_inplace_math(objs, priority_vars=None):
     return variables
 
 
-def _get_priority_vars(objects, priority_arg, compat='equals'):
+def _get_priority_vars(objects, priority_arg, compat="equals"):
     """Extract the priority variable from a list of mappings.
 
     We need this method because in some cases the priority argument itself
@@ -350,8 +383,14 @@ def expand_and_merge_variables(objs, priority_arg=None):
     return variables
 
 
-def merge_coords(objs, compat='minimal', join='outer', priority_arg=None,
-                 indexes=None, fill_value=dtypes.NA):
+def merge_coords(
+    objs,
+    compat="minimal",
+    join="outer",
+    priority_arg=None,
+    indexes=None,
+    fill_value=dtypes.NA,
+):
     """Merge coordinate variables.
 
     See merge_core below for argument descriptions. This works similarly to
@@ -360,8 +399,9 @@ def merge_coords(objs, compat='minimal', join='outer', priority_arg=None,
     """
     _assert_compat_valid(compat)
     coerced = coerce_pandas_values(objs)
-    aligned = deep_align(coerced, join=join, copy=False, indexes=indexes,
-                         fill_value=fill_value)
+    aligned = deep_align(
+        coerced, join=join, copy=False, indexes=indexes, fill_value=fill_value
+    )
     expanded = expand_variable_dicts(aligned)
     priority_vars = _get_priority_vars(aligned, priority_arg, compat=compat)
     variables = merge_variables(expanded, priority_vars, compat=compat)
@@ -370,14 +410,14 @@ def merge_coords(objs, compat='minimal', join='outer', priority_arg=None,
     return variables
 
 
-def merge_data_and_coords(data, coords, compat='broadcast_equals',
-                          join='outer'):
+def merge_data_and_coords(data, coords, compat="broadcast_equals", join="outer"):
     """Used in Dataset.__init__."""
     objs = [data, coords]
     explicit_coords = coords.keys()
     indexes = dict(extract_indexes(coords))
-    return merge_core(objs, compat, join, explicit_coords=explicit_coords,
-                      indexes=indexes)
+    return merge_core(
+        objs, compat, join, explicit_coords=explicit_coords, indexes=indexes
+    )
 
 
 def extract_indexes(coords):
@@ -397,22 +437,21 @@ def assert_valid_explicit_coords(variables, dims, explicit_coords):
     for coord_name in explicit_coords:
         if coord_name in dims and variables[coord_name].dims != (coord_name,):
             raise MergeError(
-                'coordinate %s shares a name with a dataset dimension, but is '
-                'not a 1D variable along that dimension. This is disallowed '
-                'by the xarray data model.' % coord_name)
+                "coordinate %s shares a name with a dataset dimension, but is "
+                "not a 1D variable along that dimension. This is disallowed "
+                "by the xarray data model." % coord_name
+            )
 
 
 def merge_core(
     objs,
-    compat='broadcast_equals',
-    join='outer',
+    compat="broadcast_equals",
+    join="outer",
     priority_arg=None,
     explicit_coords=None,
     indexes=None,
-    fill_value=dtypes.NA
-) -> Tuple['OrderedDict[Hashable, Variable]',
-           Set[Hashable],
-           Dict[Hashable, int]]:
+    fill_value=dtypes.NA,
+) -> Tuple["OrderedDict[Hashable, Variable]", Set[Hashable], Dict[Hashable, int]]:
     """Core logic for merging labeled objects.
 
     This is not public API.
@@ -421,7 +460,7 @@ def merge_core(
     ----------
     objs : list of mappings
         All values must be convertable to labeled arrays.
-    compat : {'identical', 'equals', 'broadcast_equals', 'no_conflicts'}, optional
+    compat : {'identical', 'equals', 'broadcast_equals', 'no_conflicts', 'override'}, optional
         Compatibility checks to use when merging variables.
     join : {'outer', 'inner', 'left', 'right'}, optional
         How to combine objects with different indexes.
@@ -452,8 +491,9 @@ def merge_core(
     _assert_compat_valid(compat)
 
     coerced = coerce_pandas_values(objs)
-    aligned = deep_align(coerced, join=join, copy=False, indexes=indexes,
-                         fill_value=fill_value)
+    aligned = deep_align(
+        coerced, join=join, copy=False, indexes=indexes, fill_value=fill_value
+    )
     expanded = expand_variable_dicts(aligned)
 
     coord_names, noncoord_names = determine_coords(coerced)
@@ -474,14 +514,15 @@ def merge_core(
 
     ambiguous_coords = coord_names.intersection(noncoord_names)
     if ambiguous_coords:
-        raise MergeError('unable to determine if these variables should be '
-                         'coordinates or not in the merged result: %s'
-                         % ambiguous_coords)
+        raise MergeError(
+            "unable to determine if these variables should be "
+            "coordinates or not in the merged result: %s" % ambiguous_coords
+        )
 
     return variables, coord_names, dims
 
 
-def merge(objects, compat='no_conflicts', join='outer', fill_value=dtypes.NA):
+def merge(objects, compat="no_conflicts", join="outer", fill_value=dtypes.NA):
     """Merge any number of xarray objects into a single Dataset as variables.
 
     Parameters
@@ -489,7 +530,7 @@ def merge(objects, compat='no_conflicts', join='outer', fill_value=dtypes.NA):
     objects : Iterable[Union[xarray.Dataset, xarray.DataArray, dict]]
         Merge together all variables from these objects. If any of them are
         DataArray objects, they must have a name.
-    compat : {'identical', 'equals', 'broadcast_equals', 'no_conflicts'}, optional
+    compat : {'identical', 'equals', 'broadcast_equals', 'no_conflicts', 'override'}, optional
         String indicating how to compare variables of the same name for
         potential conflicts:
 
@@ -501,8 +542,19 @@ def merge(objects, compat='no_conflicts', join='outer', fill_value=dtypes.NA):
         - 'no_conflicts': only values which are not null in both datasets
           must be equal. The returned dataset then contains the combination
           of all non-null values.
+        - 'override': skip comparing and pick variable from first dataset
     join : {'outer', 'inner', 'left', 'right', 'exact'}, optional
-        How to combine objects with different indexes.
+        String indicating how to combine differing indexes in objects.
+
+        - 'outer': use the union of object indexes
+        - 'inner': use the intersection of object indexes
+        - 'left': use indexes from the first object with each dimension
+        - 'right': use indexes from the last object with each dimension
+        - 'exact': instead of aligning, raise `ValueError` when indexes to be
+          aligned are not equal
+        - 'override': if indexes are of same size, rewrite indexes to be
+          those of the first object with that dimension. Indexes for the same
+          dimension must have the same size in all objects.
     fill_value : scalar, optional
         Value to use for newly missing values
 
@@ -541,32 +593,42 @@ def merge(objects, compat='no_conflicts', join='outer', fill_value=dtypes.NA):
     dict_like_objects = list()
     for obj in objects:
         if not (isinstance(obj, (DataArray, Dataset, dict))):
-            raise TypeError("objects must be an iterable containing only "
-                            "Dataset(s), DataArray(s), and dictionaries.")
+            raise TypeError(
+                "objects must be an iterable containing only "
+                "Dataset(s), DataArray(s), and dictionaries."
+            )
 
         obj = obj.to_dataset() if isinstance(obj, DataArray) else obj
         dict_like_objects.append(obj)
 
-    variables, coord_names, dims = merge_core(dict_like_objects, compat, join,
-                                              fill_value=fill_value)
+    variables, coord_names, dims = merge_core(
+        dict_like_objects, compat, join, fill_value=fill_value
+    )
     # TODO: don't always recompute indexes
-    merged = Dataset._construct_direct(
-        variables, coord_names, dims, indexes=None)
+    merged = Dataset._construct_direct(variables, coord_names, dims, indexes=None)
 
     return merged
 
 
-def dataset_merge_method(dataset, other, overwrite_vars, compat, join,
-                         fill_value=dtypes.NA):
-    """Guts of the Dataset.merge method."""
+def dataset_merge_method(
+    dataset: "Dataset",
+    other: "DatasetLike",
+    overwrite_vars: Union[Hashable, Iterable[Hashable]],
+    compat: str,
+    join: str,
+    fill_value: Any,
+) -> Tuple["OrderedDict[Hashable, Variable]", Set[Hashable], Dict[Hashable, int]]:
+    """Guts of the Dataset.merge method.
+    """
 
     # we are locked into supporting overwrite_vars for the Dataset.merge
     # method due for backwards compatibility
     # TODO: consider deprecating it?
 
-    if isinstance(overwrite_vars, str):
-        overwrite_vars = set([overwrite_vars])
-    overwrite_vars = set(overwrite_vars)
+    if isinstance(overwrite_vars, Iterable) and not isinstance(overwrite_vars, str):
+        overwrite_vars = set(overwrite_vars)
+    else:
+        overwrite_vars = {overwrite_vars}
 
     if not overwrite_vars:
         objs = [dataset, other]
@@ -575,8 +637,8 @@ def dataset_merge_method(dataset, other, overwrite_vars, compat, join,
         objs = [dataset, other]
         priority_arg = 1
     else:
-        other_overwrite = OrderedDict()
-        other_no_overwrite = OrderedDict()
+        other_overwrite = OrderedDict()  # type: MutableDatasetLike
+        other_no_overwrite = OrderedDict()  # type: MutableDatasetLike
         for k, v in other.items():
             if k in overwrite_vars:
                 other_overwrite[k] = v
@@ -585,24 +647,14 @@ def dataset_merge_method(dataset, other, overwrite_vars, compat, join,
         objs = [dataset, other_no_overwrite, other_overwrite]
         priority_arg = 2
 
-    return merge_core(objs, compat, join, priority_arg=priority_arg,
-                      fill_value=fill_value)
+    return merge_core(
+        objs, compat, join, priority_arg=priority_arg, fill_value=fill_value
+    )
 
 
 def dataset_update_method(
-    dataset: 'Dataset',
-    other: Union[
-        'Dataset',
-        Mapping[Hashable, Union[
-            'DataArray',
-            Variable,
-            Tuple[Hashable, Any],
-            Tuple[Sequence[Hashable], Any],
-        ]],
-    ],
-) -> Tuple['OrderedDict[Hashable, Variable]',
-           Set[Hashable],
-           Dict[Hashable, int]]:
+    dataset: "Dataset", other: "DatasetLike"
+) -> Tuple["OrderedDict[Hashable, Variable]", Set[Hashable], Dict[Hashable, int]]:
     """Guts of the Dataset.update method.
 
     This drops a duplicated coordinates from `other` if `other` is not an
@@ -617,10 +669,12 @@ def dataset_update_method(
         for key, value in other.items():
             if isinstance(value, DataArray):
                 # drop conflicting coordinates
-                coord_names = [c for c in value.coords
-                               if c not in value.dims and c in dataset.coords]
+                coord_names = [
+                    c
+                    for c in value.coords
+                    if c not in value.dims and c in dataset.coords
+                ]
                 if coord_names:
                     other[key] = value.drop(coord_names)
 
-    return merge_core([dataset, other], priority_arg=1,
-                      indexes=dataset.indexes)
+    return merge_core([dataset, other], priority_arg=1, indexes=dataset.indexes)
