@@ -16,6 +16,7 @@ from .variable import Variable, broadcast_variables
 class BaseInterpolator:
     """Generic interpolator class for normalizing interpolation methods
     """
+
     cons_kwargs = None  # type: Dict[str, Any]
     call_kwargs = None  # type: Dict[str, Any]
     f = None  # type: Callable
@@ -25,8 +26,9 @@ class BaseInterpolator:
         return self.f(x, **self.call_kwargs)
 
     def __repr__(self):
-        return "{type}: method={method}".format(type=self.__class__.__name__,
-                                                method=self.method)
+        return "{type}: method={method}".format(
+            type=self.__class__.__name__, method=self.method
+        )
 
 
 class NumpyInterpolator(BaseInterpolator):
@@ -36,16 +38,16 @@ class NumpyInterpolator(BaseInterpolator):
     --------
     numpy.interp
     """
-    def __init__(self, xi, yi, method='linear', fill_value=None, period=None):
 
-        if method != 'linear':
-            raise ValueError(
-                'only method `linear` is valid for the NumpyInterpolator')
+    def __init__(self, xi, yi, method="linear", fill_value=None, period=None):
+
+        if method != "linear":
+            raise ValueError("only method `linear` is valid for the NumpyInterpolator")
 
         self.method = method
         self.f = np.interp
         self.cons_kwargs = {}
-        self.call_kwargs = {'period': period}
+        self.call_kwargs = {"period": period}
 
         self._xi = xi
         self._yi = yi
@@ -60,11 +62,17 @@ class NumpyInterpolator(BaseInterpolator):
             self._left = fill_value
             self._right = fill_value
         else:
-            raise ValueError('%s is not a valid fill_value' % fill_value)
+            raise ValueError("%s is not a valid fill_value" % fill_value)
 
     def __call__(self, x):
-        return self.f(x, self._xi, self._yi, left=self._left,
-                      right=self._right, **self.call_kwargs)
+        return self.f(
+            x,
+            self._xi,
+            self._yi,
+            left=self._left,
+            right=self._right,
+            **self.call_kwargs
+        )
 
 
 class ScipyInterpolator(BaseInterpolator):
@@ -74,18 +82,30 @@ class ScipyInterpolator(BaseInterpolator):
     --------
     scipy.interpolate.interp1d
     """
-    def __init__(self, xi, yi, method=None, fill_value=None,
-                 assume_sorted=True, copy=False, bounds_error=False,
-                 order=None, **kwargs):
+
+    def __init__(
+        self,
+        xi,
+        yi,
+        method=None,
+        fill_value=None,
+        assume_sorted=True,
+        copy=False,
+        bounds_error=False,
+        order=None,
+        **kwargs
+    ):
         from scipy.interpolate import interp1d
 
         if method is None:
-            raise ValueError('method is a required argument, please supply a '
-                             'valid scipy.inter1d method (kind)')
+            raise ValueError(
+                "method is a required argument, please supply a "
+                "valid scipy.inter1d method (kind)"
+            )
 
-        if method == 'polynomial':
+        if method == "polynomial":
             if order is None:
-                raise ValueError('order is required when method=polynomial')
+                raise ValueError("order is required when method=polynomial")
             method = order
 
         self.method = method
@@ -93,14 +113,21 @@ class ScipyInterpolator(BaseInterpolator):
         self.cons_kwargs = kwargs
         self.call_kwargs = {}
 
-        if fill_value is None and method == 'linear':
+        if fill_value is None and method == "linear":
             fill_value = np.nan, np.nan
         elif fill_value is None:
             fill_value = np.nan
 
-        self.f = interp1d(xi, yi, kind=self.method, fill_value=fill_value,
-                          bounds_error=False, assume_sorted=assume_sorted,
-                          copy=copy, **self.cons_kwargs)
+        self.f = interp1d(
+            xi,
+            yi,
+            kind=self.method,
+            fill_value=fill_value,
+            bounds_error=False,
+            assume_sorted=assume_sorted,
+            copy=copy,
+            **self.cons_kwargs
+        )
 
 
 class SplineInterpolator(BaseInterpolator):
@@ -110,20 +137,29 @@ class SplineInterpolator(BaseInterpolator):
     --------
     scipy.interpolate.UnivariateSpline
     """
-    def __init__(self, xi, yi, method='spline', fill_value=None, order=3,
-                 nu=0, ext=None, **kwargs):
+
+    def __init__(
+        self,
+        xi,
+        yi,
+        method="spline",
+        fill_value=None,
+        order=3,
+        nu=0,
+        ext=None,
+        **kwargs
+    ):
         from scipy.interpolate import UnivariateSpline
 
-        if method != 'spline':
-            raise ValueError(
-                'only method `spline` is valid for the SplineInterpolator')
+        if method != "spline":
+            raise ValueError("only method `spline` is valid for the SplineInterpolator")
 
         self.method = method
         self.cons_kwargs = kwargs
-        self.call_kwargs = {'nu': nu, 'ext': ext}
+        self.call_kwargs = {"nu": nu, "ext": ext}
 
         if fill_value is not None:
-            raise ValueError('SplineInterpolator does not support fill_value')
+            raise ValueError("SplineInterpolator does not support fill_value")
 
         self.f = UnivariateSpline(xi, yi, k=order, **self.cons_kwargs)
 
@@ -143,7 +179,7 @@ def _apply_over_vars_with_dim(func, self, dim=None, **kwargs):
 
 
 def get_clean_interp_index(arr, dim, use_coordinate=True):
-    '''get index to use for x values in interpolation.
+    """get index to use for x values in interpolation.
 
     If use_coordinate is True, the coordinate that shares the name of the
     dimension along which interpolation is being performed will be used as the
@@ -151,7 +187,7 @@ def get_clean_interp_index(arr, dim, use_coordinate=True):
 
     If use_coordinate is False, the x values are set as an equally spaced
     sequence.
-    '''
+    """
     if use_coordinate:
         if use_coordinate is True:
             index = arr.get_index(dim)
@@ -159,8 +195,9 @@ def get_clean_interp_index(arr, dim, use_coordinate=True):
             index = arr.coords[use_coordinate]
             if index.ndim != 1:
                 raise ValueError(
-                    'Coordinates used for interpolation must be 1D, '
-                    '%s is %dD.' % (use_coordinate, index.ndim))
+                    "Coordinates used for interpolation must be 1D, "
+                    "%s is %dD." % (use_coordinate, index.ndim)
+                )
 
         # raise if index cannot be cast to a float (e.g. MultiIndex)
         try:
@@ -168,8 +205,10 @@ def get_clean_interp_index(arr, dim, use_coordinate=True):
         except (TypeError, ValueError):
             # pandas raises a TypeError
             # xarray/nuppy raise a ValueError
-            raise TypeError('Index must be castable to float64 to support'
-                            'interpolation, got: %s' % type(index))
+            raise TypeError(
+                "Index must be castable to float64 to support"
+                "interpolation, got: %s" % type(index)
+            )
         # check index sorting now so we can skip it later
         if not (np.diff(index) > 0).all():
             raise ValueError("Index must be monotonicly increasing")
@@ -180,12 +219,13 @@ def get_clean_interp_index(arr, dim, use_coordinate=True):
     return index
 
 
-def interp_na(self, dim=None, use_coordinate=True, method='linear', limit=None,
-              **kwargs):
+def interp_na(
+    self, dim=None, use_coordinate=True, method="linear", limit=None, **kwargs
+):
     """Interpolate values according to different methods.
     """
     if dim is None:
-        raise NotImplementedError('dim is a required argument')
+        raise NotImplementedError("dim is a required argument")
 
     if limit is not None:
         valids = _get_valid_fill_mask(self, dim, limit)
@@ -196,15 +236,19 @@ def interp_na(self, dim=None, use_coordinate=True, method='linear', limit=None,
     interpolator = partial(func_interpolate_na, interp_class, **kwargs)
 
     with warnings.catch_warnings():
-        warnings.filterwarnings('ignore', 'overflow', RuntimeWarning)
-        warnings.filterwarnings('ignore', 'invalid value', RuntimeWarning)
-        arr = apply_ufunc(interpolator, index, self,
-                          input_core_dims=[[dim], [dim]],
-                          output_core_dims=[[dim]],
-                          output_dtypes=[self.dtype],
-                          dask='parallelized',
-                          vectorize=True,
-                          keep_attrs=True).transpose(*self.dims)
+        warnings.filterwarnings("ignore", "overflow", RuntimeWarning)
+        warnings.filterwarnings("ignore", "invalid value", RuntimeWarning)
+        arr = apply_ufunc(
+            interpolator,
+            index,
+            self,
+            input_core_dims=[[dim], [dim]],
+            output_core_dims=[[dim]],
+            output_dtypes=[self.dtype],
+            dask="parallelized",
+            vectorize=True,
+            keep_attrs=True,
+        ).transpose(*self.dims)
 
     if limit is not None:
         arr = arr.where(valids)
@@ -213,7 +257,7 @@ def interp_na(self, dim=None, use_coordinate=True, method='linear', limit=None,
 
 
 def func_interpolate_na(interpolator, x, y, **kwargs):
-    '''helper function to apply interpolation along 1 dimension'''
+    """helper function to apply interpolation along 1 dimension"""
     # it would be nice if this wasn't necessary, works around:
     # "ValueError: assignment destination is read-only" in assignment below
     out = y.copy()
@@ -232,7 +276,7 @@ def func_interpolate_na(interpolator, x, y, **kwargs):
 
 
 def _bfill(arr, n=None, axis=-1):
-    '''inverse of ffill'''
+    """inverse of ffill"""
     import bottleneck as bn
 
     arr = np.flip(arr, axis=axis)
@@ -245,7 +289,7 @@ def _bfill(arr, n=None, axis=-1):
 
 
 def ffill(arr, dim=None, limit=None):
-    '''forward fill missing values'''
+    """forward fill missing values"""
     import bottleneck as bn
 
     axis = arr.get_axis_num(dim)
@@ -253,36 +297,54 @@ def ffill(arr, dim=None, limit=None):
     # work around for bottleneck 178
     _limit = limit if limit is not None else arr.shape[axis]
 
-    return apply_ufunc(bn.push, arr,
-                       dask='parallelized',
-                       keep_attrs=True,
-                       output_dtypes=[arr.dtype],
-                       kwargs=dict(n=_limit, axis=axis)).transpose(*arr.dims)
+    return apply_ufunc(
+        bn.push,
+        arr,
+        dask="parallelized",
+        keep_attrs=True,
+        output_dtypes=[arr.dtype],
+        kwargs=dict(n=_limit, axis=axis),
+    ).transpose(*arr.dims)
 
 
 def bfill(arr, dim=None, limit=None):
-    '''backfill missing values'''
+    """backfill missing values"""
     axis = arr.get_axis_num(dim)
 
     # work around for bottleneck 178
     _limit = limit if limit is not None else arr.shape[axis]
 
-    return apply_ufunc(_bfill, arr,
-                       dask='parallelized',
-                       keep_attrs=True,
-                       output_dtypes=[arr.dtype],
-                       kwargs=dict(n=_limit, axis=axis)).transpose(*arr.dims)
+    return apply_ufunc(
+        _bfill,
+        arr,
+        dask="parallelized",
+        keep_attrs=True,
+        output_dtypes=[arr.dtype],
+        kwargs=dict(n=_limit, axis=axis),
+    ).transpose(*arr.dims)
 
 
 def _get_interpolator(method, vectorizeable_only=False, **kwargs):
-    '''helper function to select the appropriate interpolator class
+    """helper function to select the appropriate interpolator class
 
     returns interpolator class and keyword arguments for the class
-    '''
-    interp1d_methods = ['linear', 'nearest', 'zero', 'slinear', 'quadratic',
-                        'cubic', 'polynomial']
-    valid_methods = interp1d_methods + ['barycentric', 'krog', 'pchip',
-                                        'spline', 'akima']
+    """
+    interp1d_methods = [
+        "linear",
+        "nearest",
+        "zero",
+        "slinear",
+        "quadratic",
+        "cubic",
+        "polynomial",
+    ]
+    valid_methods = interp1d_methods + [
+        "barycentric",
+        "krog",
+        "pchip",
+        "spline",
+        "akima",
+    ]
 
     has_scipy = True
     try:
@@ -291,83 +353,90 @@ def _get_interpolator(method, vectorizeable_only=False, **kwargs):
         has_scipy = False
 
     # prioritize scipy.interpolate
-    if (method == 'linear' and not
-            kwargs.get('fill_value', None) == 'extrapolate' and
-            not vectorizeable_only):
+    if (
+        method == "linear"
+        and not kwargs.get("fill_value", None) == "extrapolate"
+        and not vectorizeable_only
+    ):
         kwargs.update(method=method)
         interp_class = NumpyInterpolator
 
     elif method in valid_methods:
         if not has_scipy:
-            raise ImportError(
-                'Interpolation with method `%s` requires scipy' % method)
+            raise ImportError("Interpolation with method `%s` requires scipy" % method)
 
         if method in interp1d_methods:
             kwargs.update(method=method)
             interp_class = ScipyInterpolator
         elif vectorizeable_only:
-            raise ValueError('{} is not a vectorizeable interpolator. '
-                             'Available methods are {}'.format(
-                                 method, interp1d_methods))
-        elif method == 'barycentric':
+            raise ValueError(
+                "{} is not a vectorizeable interpolator. "
+                "Available methods are {}".format(method, interp1d_methods)
+            )
+        elif method == "barycentric":
             interp_class = interpolate.BarycentricInterpolator
-        elif method == 'krog':
+        elif method == "krog":
             interp_class = interpolate.KroghInterpolator
-        elif method == 'pchip':
+        elif method == "pchip":
             interp_class = interpolate.PchipInterpolator
-        elif method == 'spline':
+        elif method == "spline":
             kwargs.update(method=method)
             interp_class = SplineInterpolator
-        elif method == 'akima':
+        elif method == "akima":
             interp_class = interpolate.Akima1DInterpolator
         else:
-            raise ValueError('%s is not a valid scipy interpolator' % method)
+            raise ValueError("%s is not a valid scipy interpolator" % method)
     else:
-        raise ValueError('%s is not a valid interpolator' % method)
+        raise ValueError("%s is not a valid interpolator" % method)
 
     return interp_class, kwargs
 
 
 def _get_interpolator_nd(method, **kwargs):
-    '''helper function to select the appropriate interpolator class
+    """helper function to select the appropriate interpolator class
 
     returns interpolator class and keyword arguments for the class
-    '''
-    valid_methods = ['linear', 'nearest']
+    """
+    valid_methods = ["linear", "nearest"]
 
     try:
         from scipy import interpolate
     except ImportError:
-        raise ImportError(
-            'Interpolation with method `%s` requires scipy' % method)
+        raise ImportError("Interpolation with method `%s` requires scipy" % method)
 
     if method in valid_methods:
         kwargs.update(method=method)
         interp_class = interpolate.interpn
     else:
-        raise ValueError('%s is not a valid interpolator for interpolating '
-                         'over multiple dimensions.' % method)
+        raise ValueError(
+            "%s is not a valid interpolator for interpolating "
+            "over multiple dimensions." % method
+        )
 
     return interp_class, kwargs
 
 
 def _get_valid_fill_mask(arr, dim, limit):
-    '''helper function to determine values that can be filled when limit is not
-    None'''
+    """helper function to determine values that can be filled when limit is not
+    None"""
     kw = {dim: limit + 1}
     # we explicitly use construct method to avoid copy.
-    new_dim = utils.get_temp_dimname(arr.dims, '_window')
-    return (arr.isnull().rolling(min_periods=1, **kw)
-            .construct(new_dim, fill_value=False)
-            .sum(new_dim, skipna=False)) <= limit
+    new_dim = utils.get_temp_dimname(arr.dims, "_window")
+    return (
+        arr.isnull()
+        .rolling(min_periods=1, **kw)
+        .construct(new_dim, fill_value=False)
+        .sum(new_dim, skipna=False)
+    ) <= limit
 
 
 def _assert_single_chunk(var, axes):
     for axis in axes:
         if len(var.chunks[axis]) > 1 or var.chunks[axis][0] < var.shape[axis]:
             raise NotImplementedError(
-                'Chunking along the dimension to be interpolated '
-                '({}) is not yet supported.'.format(axis))
+                "Chunking along the dimension to be interpolated "
+                "({}) is not yet supported.".format(axis)
+            )
 
 
 def _localize(var, indexes_coords):
@@ -377,8 +446,8 @@ def _localize(var, indexes_coords):
     indexes = {}
     for dim, [x, new_x] in indexes_coords.items():
         index = x.to_index()
-        imin = index.get_loc(np.min(new_x.values), method='nearest')
-        imax = index.get_loc(np.max(new_x.values), method='nearest')
+        imin = index.get_loc(np.min(new_x.values), method="nearest")
+        imax = index.get_loc(np.max(new_x.values), method="nearest")
 
         indexes[dim] = slice(max(imin - 2, 0), imax + 2)
         indexes_coords[dim] = (x[indexes[dim]], new_x)
@@ -435,11 +504,11 @@ def interp(var, indexes_coords, method, **kwargs):
         return var.copy()
 
     # simple speed up for the local interpolation
-    if method in ['linear', 'nearest']:
+    if method in ["linear", "nearest"]:
         var, indexes_coords = _localize(var, indexes_coords)
 
     # default behavior
-    kwargs['bounds_error'] = kwargs.get('bounds_error', False)
+    kwargs["bounds_error"] = kwargs.get("bounds_error", False)
 
     # target dimensions
     dims = list(indexes_coords)
@@ -450,8 +519,9 @@ def interp(var, indexes_coords, method, **kwargs):
     broadcast_dims = [d for d in var.dims if d not in dims]
     original_dims = broadcast_dims + dims
     new_dims = broadcast_dims + list(destination[0].dims)
-    interped = interp_func(var.transpose(*original_dims).data,
-                           x, destination, method, kwargs)
+    interped = interp_func(
+        var.transpose(*original_dims).data, x, destination, method, kwargs
+    )
 
     result = Variable(new_dims, interped, attrs=var.attrs)
 
@@ -502,8 +572,7 @@ def interp_func(var, x, new_x, method, kwargs):
         return var.copy()
 
     if len(x) == 1:
-        func, kwargs = _get_interpolator(method, vectorizeable_only=True,
-                                         **kwargs)
+        func, kwargs = _get_interpolator(method, vectorizeable_only=True, **kwargs)
     else:
         func, kwargs = _get_interpolator_nd(method, **kwargs)
 
@@ -511,12 +580,21 @@ def interp_func(var, x, new_x, method, kwargs):
         import dask.array as da
 
         _assert_single_chunk(var, range(var.ndim - len(x), var.ndim))
-        chunks = var.chunks[:-len(x)] + new_x[0].shape
+        chunks = var.chunks[: -len(x)] + new_x[0].shape
         drop_axis = range(var.ndim - len(x), var.ndim)
         new_axis = range(var.ndim - len(x), var.ndim - len(x) + new_x[0].ndim)
-        return da.map_blocks(_interpnd, var, x, new_x, func, kwargs,
-                             dtype=var.dtype, chunks=chunks,
-                             new_axis=new_axis, drop_axis=drop_axis)
+        return da.map_blocks(
+            _interpnd,
+            var,
+            x,
+            new_x,
+            func,
+            kwargs,
+            dtype=var.dtype,
+            chunks=chunks,
+            new_axis=new_axis,
+            drop_axis=drop_axis,
+        )
 
     return _interpnd(var, x, new_x, func, kwargs)
 
