@@ -1,7 +1,7 @@
 from . import ops
-from .groupby import DEFAULT_DIMS, DataArrayGroupBy, DatasetGroupBy
+from .groupby import DataArrayGroupBy, DatasetGroupBy
 
-RESAMPLE_DIM = '__resample_dim__'
+RESAMPLE_DIM = "__resample_dim__"
 
 
 class Resample:
@@ -49,27 +49,28 @@ class Resample:
             if self._dim in v.dims:
                 self._obj = self._obj.drop(k)
 
-        if method == 'asfreq':
+        if method == "asfreq":
             return self.mean(self._dim)
 
-        elif method in ['pad', 'ffill', 'backfill', 'bfill', 'nearest']:
+        elif method in ["pad", "ffill", "backfill", "bfill", "nearest"]:
             kwargs = kwargs.copy()
             kwargs.update(**{self._dim: upsampled_index})
             return self._obj.reindex(method=method, *args, **kwargs)
 
-        elif method == 'interpolate':
+        elif method == "interpolate":
             return self._interpolate(*args, **kwargs)
 
         else:
-            raise ValueError('Specified method was "{}" but must be one of'
-                             '"asfreq", "ffill", "bfill", or "interpolate"'
-                             .format(method))
+            raise ValueError(
+                'Specified method was "{}" but must be one of'
+                '"asfreq", "ffill", "bfill", or "interpolate"'.format(method)
+            )
 
     def asfreq(self):
         """Return values of original object at the new up-sampling frequency;
         essentially a re-index with new times set to NaN.
         """
-        return self._upsample('asfreq')
+        return self._upsample("asfreq")
 
     def pad(self, tolerance=None):
         """Forward fill new values at up-sampled frequency.
@@ -84,7 +85,8 @@ class Resample:
             new values. Data with indices that are outside the given
             tolerance are filled with ``NaN``  s
         """
-        return self._upsample('pad', tolerance=tolerance)
+        return self._upsample("pad", tolerance=tolerance)
+
     ffill = pad
 
     def backfill(self, tolerance=None):
@@ -100,7 +102,8 @@ class Resample:
             new values. Data with indices that are outside the given
             tolerance are filled with ``NaN`` s
         """
-        return self._upsample('backfill', tolerance=tolerance)
+        return self._upsample("backfill", tolerance=tolerance)
+
     bfill = backfill
 
     def nearest(self, tolerance=None):
@@ -117,9 +120,9 @@ class Resample:
             new values. Data with indices that are outside the given
             tolerance are filled with ``NaN`` s
         """
-        return self._upsample('nearest', tolerance=tolerance)
+        return self._upsample("nearest", tolerance=tolerance)
 
-    def interpolate(self, kind='linear'):
+    def interpolate(self, kind="linear"):
         """Interpolate up-sampled data using the original data
         as knots.
 
@@ -136,7 +139,7 @@ class Resample:
         """
         return self._interpolate(kind=kind)
 
-    def _interpolate(self, kind='linear'):
+    def _interpolate(self, kind="linear"):
         """Apply scipy.interpolate.interp1d along resampling dimension."""
         # drop any existing non-dimension coordinates along the resampling
         # dimension
@@ -144,9 +147,12 @@ class Resample:
         for k, v in self._obj.coords.items():
             if k != self._dim and self._dim in v.dims:
                 dummy = dummy.drop(k)
-        return dummy.interp(assume_sorted=True, method=kind,
-                            kwargs={'bounds_error': False},
-                            **{self._dim: self._full_index})
+        return dummy.interp(
+            assume_sorted=True,
+            method=kind,
+            kwargs={"bounds_error": False},
+            **{self._dim: self._full_index}
+        )
 
 
 class DataArrayResample(DataArrayGroupBy, Resample):
@@ -157,9 +163,11 @@ class DataArrayResample(DataArrayGroupBy, Resample):
     def __init__(self, *args, dim=None, resample_dim=None, **kwargs):
 
         if dim == resample_dim:
-            raise ValueError("Proxy resampling dimension ('{}') "
-                             "cannot have the same name as actual dimension "
-                             "('{}')! ".format(resample_dim, dim))
+            raise ValueError(
+                "Proxy resampling dimension ('{}') "
+                "cannot have the same name as actual dimension "
+                "('{}')! ".format(resample_dim, dim)
+            )
         self._dim = dim
         self._resample_dim = resample_dim
 
@@ -204,8 +212,7 @@ class DataArrayResample(DataArrayGroupBy, Resample):
         applied : DataArray or DataArray
             The result of splitting, applying and combining this array.
         """
-        combined = super().apply(
-            func, shortcut=shortcut, args=args, **kwargs)
+        combined = super().apply(func, shortcut=shortcut, args=args, **kwargs)
 
         # If the aggregation function didn't drop the original resampling
         # dimension, then we need to do so before we can rename the proxy
@@ -230,9 +237,11 @@ class DatasetResample(DatasetGroupBy, Resample):
     def __init__(self, *args, dim=None, resample_dim=None, **kwargs):
 
         if dim == resample_dim:
-            raise ValueError("Proxy resampling dimension ('{}') "
-                             "cannot have the same name as actual dimension "
-                             "('{}')! ".format(resample_dim, dim))
+            raise ValueError(
+                "Proxy resampling dimension ('{}') "
+                "cannot have the same name as actual dimension "
+                "('{}')! ".format(resample_dim, dim)
+            )
         self._dim = dim
         self._resample_dim = resample_dim
 
@@ -298,11 +307,7 @@ class DatasetResample(DatasetGroupBy, Resample):
             Array with summarized data and the indicated dimension(s)
             removed.
         """
-        if dim == DEFAULT_DIMS:
-            dim = None
-
-        return super().reduce(
-            func, dim, keep_attrs, **kwargs)
+        return super().reduce(func, dim, keep_attrs, **kwargs)
 
 
 ops.inject_reduce_methods(DatasetResample)
