@@ -1,5 +1,3 @@
-from __future__ import absolute_import, division, print_function
-
 import functools
 import itertools
 import warnings
@@ -7,14 +5,16 @@ import warnings
 import numpy as np
 
 from ..core.formatting import format_item
-from ..core.pycompat import getargspec
 from .utils import (
-    _determine_cmap_params, _infer_xy_labels, import_matplotlib_pyplot,
-    label_from_attrs)
+    _infer_xy_labels,
+    _process_cmap_cbar_kwargs,
+    import_matplotlib_pyplot,
+    label_from_attrs,
+)
 
 # Overrides axes.labelsize, xtick.major.size, ytick.major.size
 # from mpl.rcParams
-_FONTSIZE = 'small'
+_FONTSIZE = "small"
 # For major ticks on x, y axes
 _NTICKS = 5
 
@@ -27,12 +27,12 @@ def _nicetitle(coord, value, maxchar, template):
     title = template.format(coord=coord, value=prettyvalue)
 
     if len(title) > maxchar:
-        title = title[:(maxchar - 3)] + '...'
+        title = title[: (maxchar - 3)] + "..."
 
     return title
 
 
-class FacetGrid(object):
+class FacetGrid:
     """
     Initialize the matplotlib figure and FacetGrid object.
 
@@ -67,12 +67,21 @@ class FacetGrid(object):
         Contains dictionaries mapping coordinate names to values. None is
         used as a sentinel value for axes which should remain empty, ie.
         sometimes the bottom right grid
-
     """
 
-    def __init__(self, data, col=None, row=None, col_wrap=None,
-                 sharex=True, sharey=True, figsize=None, aspect=1, size=3,
-                 subplot_kws=None):
+    def __init__(
+        self,
+        data,
+        col=None,
+        row=None,
+        col_wrap=None,
+        sharex=True,
+        sharey=True,
+        figsize=None,
+        aspect=1,
+        size=3,
+        subplot_kws=None,
+    ):
         """
         Parameters
         ----------
@@ -106,8 +115,10 @@ class FacetGrid(object):
         rep_col = col is not None and not data[col].to_index().is_unique
         rep_row = row is not None and not data[row].to_index().is_unique
         if rep_col or rep_row:
-            raise ValueError('Coordinates used for faceting cannot '
-                             'contain repeated (nonunique) values.')
+            raise ValueError(
+                "Coordinates used for faceting cannot "
+                "contain repeated (nonunique) values."
+            )
 
         # single_group is the grouping variable, if there is exactly one
         if col and row:
@@ -116,15 +127,13 @@ class FacetGrid(object):
             ncol = len(data[col])
             nfacet = nrow * ncol
             if col_wrap is not None:
-                warnings.warn('Ignoring col_wrap since both col and row '
-                              'were passed')
+                warnings.warn("Ignoring col_wrap since both col and row " "were passed")
         elif row and not col:
             single_group = row
         elif not row and col:
             single_group = col
         else:
-            raise ValueError(
-                'Pass a coordinate name as an argument for row or col')
+            raise ValueError("Pass a coordinate name as an argument for row or col")
 
         # Compute grid shape
         if single_group:
@@ -148,17 +157,22 @@ class FacetGrid(object):
             cbar_space = 1
             figsize = (ncol * size * aspect + cbar_space, nrow * size)
 
-        fig, axes = plt.subplots(nrow, ncol,
-                                 sharex=sharex, sharey=sharey, squeeze=False,
-                                 figsize=figsize, subplot_kw=subplot_kws)
+        fig, axes = plt.subplots(
+            nrow,
+            ncol,
+            sharex=sharex,
+            sharey=sharey,
+            squeeze=False,
+            figsize=figsize,
+            subplot_kw=subplot_kws,
+        )
 
         # Set up the lists of names for the row and column facet variables
         col_names = list(data[col].values) if col else []
         row_names = list(data[row].values) if row else []
 
         if single_group:
-            full = [{single_group: x} for x in
-                    data[single_group].values]
+            full = [{single_group: x} for x in data[single_group].values]
             empty = [None for x in range(nrow * ncol - len(full))]
             name_dicts = full + empty
         else:
@@ -177,6 +191,7 @@ class FacetGrid(object):
         self.axes = axes
         self.row_names = row_names
         self.col_names = col_names
+        self.figlegend = None
 
         # Next the private variables
         self._single_group = single_group
@@ -221,88 +236,71 @@ class FacetGrid(object):
 
         """
 
-        cmapkw = kwargs.get('cmap')
-        colorskw = kwargs.get('colors')
-        cbar_kwargs = kwargs.pop('cbar_kwargs', {})
-        cbar_kwargs = {} if cbar_kwargs is None else dict(cbar_kwargs)
+        if kwargs.get("cbar_ax", None) is not None:
+            raise ValueError("cbar_ax not supported by FacetGrid.")
 
-        if kwargs.get('cbar_ax', None) is not None:
-            raise ValueError('cbar_ax not supported by FacetGrid.')
+        cmap_params, cbar_kwargs = _process_cmap_cbar_kwargs(
+            func, self.data.values, **kwargs
+        )
 
-        # colors is mutually exclusive with cmap
-        if cmapkw and colorskw:
-            raise ValueError("Can't specify both cmap and colors.")
-
-        # These should be consistent with xarray.plot._plot2d
-        cmap_kwargs = {'plot_data': self.data.values,
-                       # MPL default
-                       'levels': 7 if 'contour' in func.__name__ else None,
-                       'filled': func.__name__ != 'contour',
-                       }
-
-        cmap_args = getargspec(_determine_cmap_params).args
-        cmap_kwargs.update((a, kwargs[a]) for a in cmap_args if a in kwargs)
-
-        cmap_params = _determine_cmap_params(**cmap_kwargs)
-
-        if colorskw is not None:
-            cmap_params['cmap'] = None
+        self._cmap_extend = cmap_params.get("extend")
 
         # Order is important
-        func_kwargs = kwargs.copy()
+        func_kwargs = {
+            k: v
+            for k, v in kwargs.items()
+            if k not in {"cmap", "colors", "cbar_kwargs", "levels"}
+        }
         func_kwargs.update(cmap_params)
-        func_kwargs.update({'add_colorbar': False, 'add_labels': False})
+        func_kwargs.update({"add_colorbar": False, "add_labels": False})
 
         # Get x, y labels for the first subplot
         x, y = _infer_xy_labels(
-            darray=self.data.loc[self.name_dicts.flat[0]], x=x, y=y,
-            imshow=func.__name__ == 'imshow', rgb=kwargs.get('rgb', None))
+            darray=self.data.loc[self.name_dicts.flat[0]],
+            x=x,
+            y=y,
+            imshow=func.__name__ == "imshow",
+            rgb=kwargs.get("rgb", None),
+        )
 
         for d, ax in zip(self.name_dicts.flat, self.axes.flat):
             # None is the sentinel value
             if d is not None:
                 subset = self.data.loc[d]
-                mappable = func(subset, x, y, ax=ax, **func_kwargs)
+                mappable = func(subset, x=x, y=y, ax=ax, **func_kwargs)
                 self._mappables.append(mappable)
 
-        self._cmap_extend = cmap_params.get('extend')
         self._finalize_grid(x, y)
 
-        if kwargs.get('add_colorbar', True):
+        if kwargs.get("add_colorbar", True):
             self.add_colorbar(**cbar_kwargs)
 
         return self
 
-    def map_dataarray_line(self, x=None, y=None, hue=None, **kwargs):
-        """
-        Apply a line plot to a 2d facet subset of the data.
-
-        Parameters
-        ----------
-        x, y, hue: string
-            dimension names for the axes and hues of each facet
-
-        Returns
-        -------
-        self : FacetGrid object
-
-        """
-        from .plot import line, _infer_line_data
-
-        add_legend = kwargs.pop('add_legend', True)
-        kwargs['add_legend'] = False
+    def map_dataarray_line(
+        self, func, x, y, hue, add_legend=True, _labels=None, **kwargs
+    ):
+        from .plot import _infer_line_data
 
         for d, ax in zip(self.name_dicts.flat, self.axes.flat):
             # None is the sentinel value
             if d is not None:
                 subset = self.data.loc[d]
-                mappable = line(subset, x=x, y=y, hue=hue,
-                                ax=ax, _labels=False,
-                                **kwargs)
+                mappable = func(
+                    subset,
+                    x=x,
+                    y=y,
+                    ax=ax,
+                    hue=hue,
+                    add_legend=False,
+                    _labels=False,
+                    **kwargs
+                )
                 self._mappables.append(mappable)
+
         _, _, hueplt, xlabel, ylabel, huelabel = _infer_line_data(
-            darray=self.data.loc[self.name_dicts.flat[0]],
-            x=x, y=y, hue=hue)
+            darray=self.data.loc[self.name_dicts.flat[0]], x=x, y=y, hue=hue
+        )
 
         self._hue_var = hueplt
         self._hue_label = huelabel
@@ -313,6 +311,52 @@ class FacetGrid(object):
 
         return self
 
+    def map_dataset(
+        self, func, x=None, y=None, hue=None, hue_style=None, add_guide=None, **kwargs
+    ):
+        from .dataset_plot import _infer_meta_data, _parse_size
+
+        kwargs["add_guide"] = False
+        kwargs["_is_facetgrid"] = True
+
+        if kwargs.get("markersize", None):
+            kwargs["size_mapping"] = _parse_size(
+                self.data[kwargs["markersize"]], kwargs.pop("size_norm", None)
+            )
+
+        meta_data = _infer_meta_data(self.data, x, y, hue, hue_style, add_guide)
+        kwargs["meta_data"] = meta_data
+
+        if hue and meta_data["hue_style"] == "continuous":
+            cmap_params, cbar_kwargs = _process_cmap_cbar_kwargs(
+                func, self.data[hue].values, **kwargs
+            )
+            kwargs["meta_data"]["cmap_params"] = cmap_params
+            kwargs["meta_data"]["cbar_kwargs"] = cbar_kwargs
+
+        for d, ax in zip(self.name_dicts.flat, self.axes.flat):
+            # None is the sentinel value
+            if d is not None:
+                subset = self.data.loc[d]
+                maybe_mappable = func(
+                    ds=subset, x=x, y=y, hue=hue, hue_style=hue_style, ax=ax, **kwargs
+                )
+                # TODO: this is needed to get legends to work.
+                # but maybe_mappable is a list in that case :/
+                self._mappables.append(maybe_mappable)
+
+        self._finalize_grid(meta_data["xlabel"], meta_data["ylabel"])
+
+        if hue:
+            self._hue_label = meta_data.pop("hue_label", None)
+            if meta_data["add_legend"]:
+                self._hue_var = meta_data["hue"]
+                self.add_legend()
+            elif meta_data["add_colorbar"]:
+                self.add_colorbar(label=self._hue_label, **cbar_kwargs)
+
+        return self
+
     def _finalize_grid(self, *axlabels):
         """Finalize the annotations and layout."""
         if not self._finalized:
@@ -331,8 +375,11 @@ class FacetGrid(object):
             handles=self._mappables[-1],
             labels=list(self._hue_var.values),
             title=self._hue_label,
-            loc="center right", **kwargs)
+            loc="center right",
+            **kwargs
+        )
 
+        self.figlegend = figlegend
         # Draw the plot to set the bounding boxes correctly
         self.fig.draw(self.fig.canvas.get_renderer())
 
@@ -359,12 +406,12 @@ class FacetGrid(object):
         """
         kwargs = kwargs.copy()
         if self._cmap_extend is not None:
-            kwargs.setdefault('extend', self._cmap_extend)
-        if 'label' not in kwargs:
-            kwargs.setdefault('label', label_from_attrs(self.data))
-        self.cbar = self.fig.colorbar(self._mappables[-1],
-                                      ax=list(self.axes.flat),
-                                      **kwargs)
+            kwargs.setdefault("extend", self._cmap_extend)
+        if "label" not in kwargs:
+            kwargs.setdefault("label", label_from_attrs(self.data))
+        self.cbar = self.fig.colorbar(
+            self._mappables[-1], ax=list(self.axes.flat), **kwargs
+        )
         return self
 
     def set_axis_labels(self, x_var=None, y_var=None):
@@ -401,8 +448,7 @@ class FacetGrid(object):
             ax.set_ylabel(label, **kwargs)
         return self
 
-    def set_titles(self, template="{coord} = {value}", maxchar=30,
-                   **kwargs):
+    def set_titles(self, template="{coord} = {value}", maxchar=30, size=None, **kwargs):
         """
         Draw titles either above each facet or on the grid margins.
 
@@ -422,10 +468,10 @@ class FacetGrid(object):
         """
         import matplotlib as mpl
 
-        kwargs["size"] = kwargs.pop("size", mpl.rcParams["axes.labelsize"])
+        if size is None:
+            size = mpl.rcParams["axes.labelsize"]
 
-        nicetitle = functools.partial(_nicetitle, maxchar=maxchar,
-                                      template=template)
+        nicetitle = functools.partial(_nicetitle, maxchar=maxchar, template=template)
 
         if self._single_group:
             for d, ax in zip(self.name_dicts.flat, self.axes.flat):
@@ -433,25 +479,29 @@ class FacetGrid(object):
                 if d is not None:
                     coord, value = list(d.items()).pop()
                     title = nicetitle(coord, value, maxchar=maxchar)
-                    ax.set_title(title, **kwargs)
+                    ax.set_title(title, size=size, **kwargs)
         else:
             # The row titles on the right edge of the grid
             for ax, row_name in zip(self.axes[:, -1], self.row_names):
-                title = nicetitle(coord=self._row_var, value=row_name,
-                                  maxchar=maxchar)
-                ax.annotate(title, xy=(1.02, .5), xycoords="axes fraction",
-                            rotation=270, ha="left", va="center", **kwargs)
+                title = nicetitle(coord=self._row_var, value=row_name, maxchar=maxchar)
+                ax.annotate(
+                    title,
+                    xy=(1.02, 0.5),
+                    xycoords="axes fraction",
+                    rotation=270,
+                    ha="left",
+                    va="center",
+                    **kwargs
+                )
 
             # The column titles on the top row
             for ax, col_name in zip(self.axes[0, :], self.col_names):
-                title = nicetitle(coord=self._col_var, value=col_name,
-                                  maxchar=maxchar)
-                ax.set_title(title, **kwargs)
+                title = nicetitle(coord=self._col_var, value=col_name, maxchar=maxchar)
+                ax.set_title(title, size=size, **kwargs)
 
         return self
 
-    def set_ticks(self, max_xticks=_NTICKS, max_yticks=_NTICKS,
-                  fontsize=_FONTSIZE):
+    def set_ticks(self, max_xticks=_NTICKS, max_yticks=_NTICKS, fontsize=_FONTSIZE):
         """
         Set and control tick behavior
 
@@ -476,9 +526,10 @@ class FacetGrid(object):
         for ax in self.axes.flat:
             ax.xaxis.set_major_locator(x_major_locator)
             ax.yaxis.set_major_locator(y_major_locator)
-            for tick in itertools.chain(ax.xaxis.get_major_ticks(),
-                                        ax.yaxis.get_major_ticks()):
-                tick.label.set_fontsize(fontsize)
+            for tick in itertools.chain(
+                ax.xaxis.get_major_ticks(), ax.yaxis.get_major_ticks()
+            ):
+                tick.label1.set_fontsize(fontsize)
 
         return self
 
@@ -515,10 +566,64 @@ class FacetGrid(object):
                 maybe_mappable = func(*innerargs, **kwargs)
                 # TODO: better way to verify that an artist is mappable?
                 # https://stackoverflow.com/questions/33023036/is-it-possible-to-detect-if-a-matplotlib-artist-is-a-mappable-suitable-for-use-w#33023522
-                if (maybe_mappable and
-                   hasattr(maybe_mappable, 'autoscale_None')):
+                if maybe_mappable and hasattr(maybe_mappable, "autoscale_None"):
                     self._mappables.append(maybe_mappable)
 
         self._finalize_grid(*args[:2])
 
         return self
+
+
+def _easy_facetgrid(
+    data,
+    plotfunc,
+    kind,
+    x=None,
+    y=None,
+    row=None,
+    col=None,
+    col_wrap=None,
+    sharex=True,
+    sharey=True,
+    aspect=None,
+    size=None,
+    subplot_kws=None,
+    ax=None,
+    figsize=None,
+    **kwargs
+):
+    """
+    Convenience method to call xarray.plot.FacetGrid from 2d plotting methods
+
+    kwargs are the arguments to 2d plotting method
+    """
+    if ax is not None:
+        raise ValueError("Can't use axes when making faceted plots.")
+    if aspect is None:
+        aspect = 1
+    if size is None:
+        size = 3
+    elif figsize is not None:
+        raise ValueError("cannot provide both `figsize` and `size` arguments")
+
+    g = FacetGrid(
+        data=data,
+        col=col,
+        row=row,
+        col_wrap=col_wrap,
+        sharex=sharex,
+        sharey=sharey,
+        figsize=figsize,
+        aspect=aspect,
+        size=size,
+        subplot_kws=subplot_kws,
+    )
+
+    if kind == "line":
+        return g.map_dataarray_line(plotfunc, x, y, **kwargs)
+
+    if kind == "dataarray":
+        return g.map_dataarray(plotfunc, x, y, **kwargs)
+
+    if kind == "dataset":
+        return g.map_dataset(plotfunc, x, y, **kwargs)
