@@ -1,17 +1,21 @@
-'''
+"""
 Useful for:
 
 * users learning xarray
 * building tutorials in the documentation.
 
-'''
+"""
 import hashlib
 import os as _os
 from urllib.request import urlretrieve
 
-from .backends.api import open_dataset as _open_dataset
+import numpy as np
 
-_default_cache_dir = _os.sep.join(('~', '.xarray_tutorial_data'))
+from .backends.api import open_dataset as _open_dataset
+from .core.dataarray import DataArray
+from .core.dataset import Dataset
+
+_default_cache_dir = _os.sep.join(("~", ".xarray_tutorial_data"))
 
 
 def file_md5_checksum(fname):
@@ -22,9 +26,14 @@ def file_md5_checksum(fname):
 
 
 # idea borrowed from Seaborn
-def open_dataset(name, cache=True, cache_dir=_default_cache_dir,
-                 github_url='https://github.com/pydata/xarray-data',
-                 branch='master', **kws):
+def open_dataset(
+    name,
+    cache=True,
+    cache_dir=_default_cache_dir,
+    github_url="https://github.com/pydata/xarray-data",
+    branch="master",
+    **kws
+):
     """
     Open a dataset from the online repository (requires internet).
 
@@ -52,9 +61,9 @@ def open_dataset(name, cache=True, cache_dir=_default_cache_dir,
 
     """
     longdir = _os.path.expanduser(cache_dir)
-    fullname = name + '.nc'
+    fullname = name + ".nc"
     localfile = _os.sep.join((longdir, fullname))
-    md5name = name + '.md5'
+    md5name = name + ".md5"
     md5file = _os.sep.join((longdir, md5name))
 
     if not _os.path.exists(localfile):
@@ -64,20 +73,20 @@ def open_dataset(name, cache=True, cache_dir=_default_cache_dir,
         if not _os.path.isdir(longdir):
             _os.mkdir(longdir)
 
-        url = '/'.join((github_url, 'raw', branch, fullname))
+        url = "/".join((github_url, "raw", branch, fullname))
         urlretrieve(url, localfile)
-        url = '/'.join((github_url, 'raw', branch, md5name))
+        url = "/".join((github_url, "raw", branch, md5name))
         urlretrieve(url, md5file)
 
         localmd5 = file_md5_checksum(localfile)
-        with open(md5file, 'r') as f:
+        with open(md5file, "r") as f:
             remotemd5 = f.read()
         if localmd5 != remotemd5:
             _os.remove(localfile)
             msg = """
             MD5 checksum does not match, try downloading dataset again.
             """
-            raise IOError(msg)
+            raise OSError(msg)
 
     ds = _open_dataset(localfile, **kws)
 
@@ -99,3 +108,30 @@ def load_dataset(*args, **kwargs):
     """
     with open_dataset(*args, **kwargs) as ds:
         return ds.load()
+
+
+def scatter_example_dataset():
+    A = DataArray(
+        np.zeros([3, 11, 4, 4]),
+        dims=["x", "y", "z", "w"],
+        coords=[
+            np.arange(3),
+            np.linspace(0, 1, 11),
+            np.arange(4),
+            0.1 * np.random.randn(4),
+        ],
+    )
+    B = 0.1 * A.x ** 2 + A.y ** 2.5 + 0.1 * A.z * A.w
+    A = -0.1 * A.x + A.y / (5 + A.z) + A.w
+    ds = Dataset({"A": A, "B": B})
+    ds["w"] = ["one", "two", "three", "five"]
+
+    ds.x.attrs["units"] = "xunits"
+    ds.y.attrs["units"] = "yunits"
+    ds.z.attrs["units"] = "zunits"
+    ds.w.attrs["units"] = "wunits"
+
+    ds.A.attrs["units"] = "Aunits"
+    ds.B.attrs["units"] = "Bunits"
+
+    return ds
