@@ -1,5 +1,4 @@
 """Utility functions for printing version information."""
-import codecs
 import importlib
 import locale
 import os
@@ -18,9 +17,11 @@ def get_sys_info():
     commit = None
     if os.path.isdir(".git") and os.path.isdir("xarray"):
         try:
-            pipe = subprocess.Popen('git log --format="%H" -n 1'.split(" "),
-                                    stdout=subprocess.PIPE,
-                                    stderr=subprocess.PIPE)
+            pipe = subprocess.Popen(
+                'git log --format="%H" -n 1'.split(" "),
+                stdout=subprocess.PIPE,
+                stderr=subprocess.PIPE,
+            )
             so, serr = pipe.communicate()
         except Exception:
             pass
@@ -28,30 +29,30 @@ def get_sys_info():
             if pipe.returncode == 0:
                 commit = so
                 try:
-                    commit = so.decode('utf-8')
+                    commit = so.decode("utf-8")
                 except ValueError:
                     pass
                 commit = commit.strip().strip('"')
 
-    blob.append(('commit', commit))
+    blob.append(("commit", commit))
 
     try:
-        (sysname, nodename, release,
-         version, machine, processor) = platform.uname()
-        blob.extend([
-            ("python", sys.version),
-            ("python-bits", struct.calcsize("P") * 8),
-            ("OS", "%s" % (sysname)),
-            ("OS-release", "%s" % (release)),
-            # ("Version", "%s" % (version)),
-            ("machine", "%s" % (machine)),
-            ("processor", "%s" % (processor)),
-            ("byteorder", "%s" % sys.byteorder),
-            ("LC_ALL", "%s" % os.environ.get('LC_ALL', "None")),
-            ("LANG", "%s" % os.environ.get('LANG', "None")),
-            ("LOCALE", "%s.%s" % locale.getlocale()),
-
-        ])
+        (sysname, nodename, release, version, machine, processor) = platform.uname()
+        blob.extend(
+            [
+                ("python", sys.version),
+                ("python-bits", struct.calcsize("P") * 8),
+                ("OS", "%s" % (sysname)),
+                ("OS-release", "%s" % (release)),
+                # ("Version", "%s" % (version)),
+                ("machine", "%s" % (machine)),
+                ("processor", "%s" % (processor)),
+                ("byteorder", "%s" % sys.byteorder),
+                ("LC_ALL", "%s" % os.environ.get("LC_ALL", "None")),
+                ("LANG", "%s" % os.environ.get("LANG", "None")),
+                ("LOCALE", "%s.%s" % locale.getlocale()),
+            ]
+        )
     except Exception:
         pass
 
@@ -63,21 +64,26 @@ def netcdf_and_hdf5_versions():
     libnetcdf_version = None
     try:
         import netCDF4
+
         libhdf5_version = netCDF4.__hdf5libversion__
         libnetcdf_version = netCDF4.__netcdf4libversion__
     except ImportError:
         try:
             import h5py
-            libhdf5_version = h5py.__hdf5libversion__
+
+            libhdf5_version = h5py.version.hdf5_version
         except ImportError:
             pass
-    return [('libhdf5', libhdf5_version), ('libnetcdf', libnetcdf_version)]
+    return [("libhdf5", libhdf5_version), ("libnetcdf", libnetcdf_version)]
 
 
 def show_versions(file=sys.stdout):
     sys_info = get_sys_info()
 
-    sys_info.extend(netcdf_and_hdf5_versions())
+    try:
+        sys_info.extend(netcdf_and_hdf5_versions())
+    except Exception as e:
+        print("Error collecting netcdf / hdf5 version: {}".format(e))
 
     deps = [
         # (MODULE_NAME, f(mod) -> mod version)
@@ -129,7 +135,7 @@ def show_versions(file=sys.stdout):
                 ver = ver_f(mod)
                 deps_blob.append((modname, ver))
             except Exception:
-                deps_blob.append((modname, 'installed'))
+                deps_blob.append((modname, "installed"))
 
     print("\nINSTALLED VERSIONS", file=file)
     print("------------------", file=file)
@@ -142,5 +148,5 @@ def show_versions(file=sys.stdout):
         print("%s: %s" % (k, stat), file=file)
 
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     show_versions()
